@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #ifndef _PDF_DEFINES_H_
@@ -51,6 +64,8 @@
 #define PODOFO_MAKE_VERSION_STR(M,m,p) PODOFO_MAKE_VERSION_STR_REAL(M,m,p)
 #define PODOFO_VERSION_STR PODOFO_MAKE_VERSION_STR(PODOFO_VERSION_MAJOR, PODOFO_VERSION_MINOR, PODOFO_VERSION_PATCH)
 
+#ifndef PODOFO_COMPILE_RC
+
 // Include common system files
 // (most are now pulled in my PdfCompilerCompat.h)
 #include <wchar.h>
@@ -217,6 +232,7 @@ enum EPdfFontType {
     ePdfFontType_Type1Pfa,
     ePdfFontType_Type1Pfb,
 	ePdfFontType_Type1Base14,
+    ePdfFontType_Type3,
     ePdfFontType_Unknown = 0xff
 };
 
@@ -230,9 +246,25 @@ enum EPdfColorSpace {
     ePdfColorSpace_DeviceCMYK,        /**< CMYK */
     ePdfColorSpace_Separation,        /**< Separation */
     ePdfColorSpace_CieLab,            /**< CIE-Lab */
+    ePdfColorSpace_Indexed,           /**< Indexed */
     ePdfColorSpace_Unknown = 0xff
 };
 
+/**
+ * Enum for text rendering mode (Tr)
+ */
+enum EPdfTextRenderingMode {
+    ePdfTextRenderingMode_Fill = 0,                 /**< Default mode, fill text */
+    ePdfTextRenderingMode_Stroke,                   /**< Stroke text */
+    ePdfTextRenderingMode_FillAndStroke,            /**< Fill, then stroke text */
+    ePdfTextRenderingMode_Invisible,                /**< Neither fill nor stroke text (invisible) */
+    ePdfTextRenderingMode_FillToClipPath,           /**< Fill text and add to path for clipping */
+    ePdfTextRenderingMode_StrokeToClipPath,         /**< Stroke text and add to path for clipping */
+    ePdfTextRenderingMode_FillAndStrokeToClipPath,  /**< Fill, then stroke text and add to path for clipping */
+    ePdfTextRenderingMode_ToClipPath,               /**< Add text to path for clipping */
+    ePdfTextRenderingMode_Unknown = 0xff
+};
+
 /**
  * Enum for the different stroke styles that can be set
  * when drawing to a PDF file (mostly for line drawing).
@@ -246,6 +278,19 @@ enum EPdfStrokeStyle {
     ePdfStrokeStyle_Custom 
 };
 
+/**
+ * Enum for predefined tiling patterns.
+ */
+enum EPdfTilingPatternType {
+	ePdfTilingPatternType_BDiagonal = 1,
+	ePdfTilingPatternType_Cross,
+	ePdfTilingPatternType_DiagCross,
+	ePdfTilingPatternType_FDiagonal,
+	ePdfTilingPatternType_Horizontal,
+	ePdfTilingPatternType_Vertical,
+   ePdfTilingPatternType_Image
+};
+
 /**
  * Enum for line cap styles when drawing.
  */
@@ -494,4 +539,6 @@ template <typename T> const T PDF_MIN ( const T a, const T b ) {
  *
  */
 
+#endif // !PODOFO_COMPILE_RC
+
 #endif // _PDF_DEFINES_H_
