@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 // PdfError.h doesn't, and can't, include PdfDefines.h so we do so here.
@@ -89,7 +102,6 @@ PdfError::PdfError( const EPdfError & eCode, const char* pszFile, int line,
 }
 
 PdfError::PdfError( const PdfError & rhs )
-    : std::exception()
 {
     this->operator=( rhs );
 }
@@ -169,7 +181,7 @@ void PdfError::PrintErrorMsg() const
     PdfError::LogErrorMessage( eLogSeverity_Error, "\n\n" );
 }
 
-const char* PdfError::what() const throw()
+const char* PdfError::what() const
 {
     return PdfError::ErrorName( m_error );
 }
@@ -321,6 +333,9 @@ const char* PdfError::ErrorName( EPdfError eCode )
         case ePdfError_NotImplemented:
             pszMsg = "ePdfError_NotImplemented";
             break;
+        case ePdfError_NotCompiled:
+            pszMsg = "ePdfError_NotCompiled";
+            break;
         case ePdfError_DestinationAlreadyPresent:
             pszMsg = "ePdfError_DestinationAlreadyPresent"; 
             break;
@@ -460,6 +475,9 @@ const char* PdfError::ErrorMessage( EPdfError eCode )
         case ePdfError_NotImplemented:
             pszMsg = "This feature is currently not implemented.";
             break;
+        case ePdfError_NotCompiled:
+            pszMsg = "This feature was disabled during compile time.";
+            break;
         case ePdfError_Unknown:
             pszMsg = "Error code unknown.";
             break;
