@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfParser.h"
@@ -38,6 +51,7 @@
 #include <sstream>
 #include <algorithm>
 #include <iostream>
+#include <limits>
 
 using std::cerr;
 using std::endl;
@@ -49,6 +63,8 @@ using std::flush;
 
 namespace PoDoFo {
 
+long PdfParser::s_nMaxObjects = std::numeric_limits<long>::max();
+
 PdfParser::PdfParser( PdfVecObjects* pVecObjects )
     : PdfTokenizer(), m_vecObjects( pVecObjects )
 {
@@ -122,6 +138,7 @@ void PdfParser::Init()
     m_bStrictParsing  = false;
     m_bIgnoreBrokenObjects = false;
     m_nIncrementalUpdates = 0;
+    m_nReadNextTrailerLevel = 0;
 }
 
 void PdfParser::ParseFile( const char* pszFilename, bool bLoadOnDemand )
@@ -234,7 +251,9 @@ void PdfParser::Clear()
 void PdfParser::ReadDocumentStructure()
 {
 // Ulrich Arnold 8.9.2009, deactivated because of problems during reading xref's
-//    HasLinearizationDict();
+   //begin L.K - na test povoleno
+    HasLinearizationDict();
+    //begin L.K
     
     // position at the end of the file to search the xref table.
     m_device.Device()->Seek( 0, std::ios_base::end );
@@ -291,6 +310,11 @@ void PdfParser::ReadDocumentStructure()
         m_nNumObjects = 0;
     }
 
+    // allow caller to specify a max object count to avoid very slow load times on large documents
+    if (s_nMaxObjects != std::numeric_limits<long>::max()
+        && m_nNumObjects > s_nMaxObjects)
+        PODOFO_RAISE_ERROR_INFO( ePdfError_ValueOutOfRange,  "m_nNumObjects is greater than m_nMaxObjects." );
+
     if (m_nNumObjects > 0)
         m_offsets.resize(m_nNumObjects);
 
@@ -367,7 +391,10 @@ void PdfParser::HasLinearizationDict()
         return; // Ignore Error Code: ERROR_PDF_NO_TRAILER;
     }
 
-    char * pszObj = strstr( m_buffer.GetBuffer(), "obj" );
+    //begin L.K
+    //char * pszObj = strstr( m_buffer.GetBuffer(), "obj" );
+    char * pszObj = strstr( linearizeBuffer.GetBuffer(), "obj" );
+    //end L.K
     if( !pszObj )
         // strange that there is no obj in the first 1024 bytes,
         // but ignore it
@@ -392,13 +419,13 @@ void PdfParser::HasLinearizationDict()
             return;
         }
     } catch( PdfError & e ) {
-        PdfError::LogMessage( eLogSeverity_Warning, e.what() );
+        PdfError::LogMessage( eLogSeverity_Warning, e.ErrorName(e.GetError()) );
         delete m_pLinearization;
         m_pLinearization = NULL;
         return;
     }
     
-    long long      lXRef      = -1;
+    pdf_int64      lXRef      = -1;
     lXRef = m_pLinearization->GetDictionary().GetKeyAsLong( "T", lXRef );
     if( lXRef == -1 )
     {
@@ -483,6 +510,18 @@ void PdfParser::MergeTrailer( const PdfObject* pTrailer )
 
 void PdfParser::ReadNextTrailer()
 {
+    // be careful changing this limit - overflow limits depend on the OS, linker settings, and how much stack space compiler allocates
+    // 500 limit prevents overflow on Win7 with VC++ 2005 with default linker stack size (1000 caused overflow with same compiler/OS)
+    const int maxReadNextTrailerLevel = 500;
+    
+    ++m_nReadNextTrailerLevel;
+    
+    if ( m_nReadNextTrailerLevel > maxReadNextTrailerLevel )
+    {
+        // avoid stack overflow on documents that have circular cross references in trailer
+        PODOFO_RAISE_ERROR( ePdfError_InvalidXRef );
+    }
+
     // ReadXRefcontents has read the first 't' from "trailer" so just check for "railer"
     if( this->IsNextToken( "trailer" ) )
     //if( strcmp( m_buffer.GetBuffer(), "railer" ) == 0 )
@@ -499,6 +538,21 @@ void PdfParser::ReadNextTrailer()
         // now merge the information of this trailer with the main documents trailer
         MergeTrailer( &trailer );
         
+        if( trailer.GetDictionary().HasKey( "XRefStm" ) )
+        {
+            // Whenever we read a XRefStm key, 
+            // we know that the file was updated.
+            if( !trailer.GetDictionary().HasKey( "Prev" ) )
+                m_nIncrementalUpdates++;
+
+            try {
+                ReadXRefStreamContents( static_cast<pdf_long>(trailer.GetDictionary().GetKeyAsLong( "XRefStm", 0 )), false );
+            } catch( PdfError & e ) {
+                e.AddToCallstack( __FILE__, __LINE__, "Unable to load /XRefStm xref stream." );
+                throw e;
+            }
+        }
+
         if( trailer.GetDictionary().HasKey( "Prev" ) )
         {
             // Whenever we read a Prev key, 
@@ -517,6 +571,8 @@ void PdfParser::ReadNextTrailer()
     {
         PODOFO_RAISE_ERROR( ePdfError_NoTrailer );
     }
+
+    --m_nReadNextTrailerLevel;
 }
 
 void PdfParser::ReadTrailer()
@@ -565,16 +621,25 @@ void PdfParser::ReadXRef( pdf_long* pXRefOffset )
 
     if( !this->IsNextToken( "startxref" ) )
     {
-        PODOFO_RAISE_ERROR( ePdfError_NoXRef );
+		// Could be non-standard startref
+		if(!m_bStrictParsing) {
+			FindToken( "startref", PDF_XREF_BUF );
+			if( !this->IsNextToken( "startref" ) )
+			{
+				PODOFO_RAISE_ERROR( ePdfError_NoXRef );
+			}
+		} else 
+		{
+			PODOFO_RAISE_ERROR( ePdfError_NoXRef );
+		}
     }
-    
     *pXRefOffset = this->GetNextNumber();
 }
 
 void PdfParser::ReadXRefContents( pdf_long lOffset, bool bPositionAtEnd )
 {
-    long long nFirstObject = 0;
-    long long nNumObjects  = 0;
+    pdf_int64 nFirstObject = 0;
+    pdf_int64 nNumObjects  = 0;
 
     size_t curPosition = m_device.Device()->Tell();
     m_device.Device()->Seek(0,std::ios_base::end);
@@ -584,7 +649,8 @@ void PdfParser::ReadXRefContents( pdf_long lOffset, bool bPositionAtEnd )
     if (lOffset > fileSize)
     { 
         // Invalid "startxref" Peter Petrov 23 December 2008
-        FindToken( "startxref", PDF_XREF_BUF );
+		 // ignore returned value and get offset from the device
+        ReadXRef( &lOffset );
         lOffset = m_device.Device()->Tell();
         // TODO: hard coded value "4"
         m_buffer.Resize(PDF_XREF_BUF*4);
@@ -638,7 +704,7 @@ void PdfParser::ReadXRefContents( pdf_long lOffset, bool bPositionAtEnd )
             nNumObjects  = this->GetNextNumber();
 
 #ifdef PODOFO_VERBOSE_DEBUG
-            PdfError::DebugMessage("Reading numbers: %lli %lli\n", nFirstObject, nNumObjects );
+            PdfError::DebugMessage("Reading numbers: %" PDF_FORMAT_INT64 " %" PDF_FORMAT_INT64 "\n", nFirstObject, nNumObjects );
 #endif // PODOFO_VERBOSE_DEBUG
 
             if( bPositionAtEnd )
@@ -675,12 +741,12 @@ void PdfParser::ReadXRefContents( pdf_long lOffset, bool bPositionAtEnd )
     }
 }
 
-void PdfParser::ReadXRefSubsection( long long & nFirstObject, long long & nNumObjects )
+void PdfParser::ReadXRefSubsection( pdf_int64 & nFirstObject, pdf_int64 & nNumObjects )
 {
     int count = 0;
 
 #ifdef PODOFO_VERBOSE_DEBUG
-    PdfError::DebugMessage("Reading XRef Section: %lli with %lli Objects.\n", nFirstObject, nNumObjects );
+    PdfError::DebugMessage("Reading XRef Section: %" PDF_FORMAT_INT64 " with %" PDF_FORMAT_INT64 " Objects.\n", nFirstObject, nNumObjects );
 #endif // PODOFO_VERBOSE_DEBUG 
 
     if ( nFirstObject + nNumObjects > m_nNumObjects )
@@ -689,8 +755,8 @@ void PdfParser::ReadXRefSubsection( long long & nFirstObject, long long & nNumOb
         // specified in the trailer if any. That's an error unless we're trying
         // to recover from a missing /Size entry.
 		PdfError::LogMessage( eLogSeverity_Warning,
-			      "There are more objects (%lli) in this XRef table than "
-			      "specified in the size key of the trailer directory (%lli)!\n",
+			      "There are more objects (%" PDF_FORMAT_INT64 ") in this XRef table than "
+			      "specified in the size key of the trailer directory (%" PDF_FORMAT_INT64 ")!\n",
 			      nFirstObject + nNumObjects, m_nNumObjects );
 
 #ifdef _WIN32
@@ -724,18 +790,18 @@ void PdfParser::ReadXRefSubsection( long long & nFirstObject, long long & nNumOb
         if( !m_offsets[objID].bParsed )
         {
             m_offsets[objID].bParsed = true;
-#ifdef _WIN64
-            sscanf( m_buffer.GetBuffer(), "%10I64d %5ld %c%c%c", 
+#if defined(_WIN64) && defined(_MSC_VER)
+            sscanf( m_buffer.GetBuffer(), "%10" PDF_FORMAT_INT64 " %5ld %c%c%c", 
                     &(m_offsets[objID].lOffset), 
                     &(m_offsets[objID].lGeneration), &(m_offsets[objID].cUsed), &empty1, &empty2 );
 #else
-            long long int tmp1;
-            long int tmp2;
-            sscanf( m_buffer.GetBuffer(), "%10lld %5ld %c%c%c", 
+            pdf_int64 tmp1;
+            pdf_int64 tmp2;
+            sscanf( m_buffer.GetBuffer(), "%10" PDF_FORMAT_INT64 " %5ld %c%c%c", 
                     &tmp1, &tmp2, &(m_offsets[objID].cUsed), &empty1, &empty2 );
 
             m_offsets[objID].lOffset = static_cast<pdf_long>(tmp1);
-            m_offsets[objID].lGeneration = tmp2;
+            m_offsets[objID].lGeneration = static_cast<long>(tmp2);
 #endif
        }
 
@@ -744,7 +810,7 @@ void PdfParser::ReadXRefSubsection( long long & nFirstObject, long long & nNumOb
 
     if( count != nNumObjects )
     {
-        PdfError::LogMessage( eLogSeverity_Warning, "Count of readobject is %i. Expected %lli.\n", count, nNumObjects );
+        PdfError::LogMessage( eLogSeverity_Warning, "Count of readobject is %i. Expected %" PDF_FORMAT_INT64 ".\n", count, nNumObjects );
         PODOFO_RAISE_ERROR( ePdfError_NoXRef );
     }
 
@@ -770,8 +836,18 @@ void PdfParser::ReadXRefStreamContents( pdf_long lOffset, bool bReadOnlyTrailer
     // Check for a previous XRef stream
     if(xrefObject.HasPrevious()) 
     {
-        m_nIncrementalUpdates++;
-        this->ReadXRefStreamContents( xrefObject.GetPreviousOffset(), bReadOnlyTrailer );
+        try {
+            m_nIncrementalUpdates++;
+            this->ReadXRefStreamContents( xrefObject.GetPreviousOffset(), bReadOnlyTrailer );
+        } catch(PdfError &e) {
+            /* Be forgiving, the error happens when an entry in XRef stream points
+               to a wrong place (offset) in the PDF file. */
+            if( e != ePdfError_NoNumber )
+            {
+                e.AddToCallstack( __FILE__, __LINE__ );
+                throw e;
+            }
+        }
     }
 }
 
@@ -901,9 +977,9 @@ void PdfParser::ReadObjects()
 
 void PdfParser::ReadObjectsInternal() 
 {
-    int              i          = 0;
-    int              nLast      = 0;
-    PdfParserObject* pObject    = NULL;
+    int              i            = 0;
+    int              nLast        = 0;
+    PdfParserObject* pObject      = NULL;
 
     // Read objects
     for( i=0; i < m_nNumObjects; i++ )
@@ -922,7 +998,17 @@ void PdfParser::ReadObjectsInternal()
             pObject = new PdfParserObject( m_vecObjects, m_device, m_buffer, m_offsets[i].lOffset );
             pObject->SetLoadOnDemand( m_bLoadOnDemand );
             try {
-                pObject->ParseFile( m_pEncrypt );
+				pObject->ParseFile( m_pEncrypt );
+				if (m_pEncrypt && pObject->IsDictionary()) {
+					PdfObject* pObjType = pObject->GetDictionary().GetKey( PdfName::KeyType );
+					if( pObjType && pObjType->IsName() && pObjType->GetName() == "XRef" ) {
+						// XRef is never encrypted
+						delete pObject;
+						pObject = new PdfParserObject( m_vecObjects, m_device, m_buffer, m_offsets[i].lOffset );
+						pObject->SetLoadOnDemand( m_bLoadOnDemand );
+						pObject->ParseFile( NULL );
+					}
+				}
                 nLast = pObject->Reference().ObjectNumber();
 
                 /*
@@ -984,7 +1070,7 @@ void PdfParser::ReadObjectsInternal()
             {
                 PdfError::LogMessage( eLogSeverity_Warning, 
                                       "Treating object %i 0 R as a free object." );
-                m_vecObjects->AddFreeObject( PdfReference( i, 1LL ) );
+                m_vecObjects->AddFreeObject( PdfReference( i, PODOFO_LL_LITERAL(1) ) );
             }
         }
 // Ulrich Arnold 30.7.2009: the linked free list in the xref section is not always correct in pdf's
@@ -995,11 +1081,11 @@ void PdfParser::ReadObjectsInternal()
 //							robustly from all places which are either free or unparsed
 //      else if( m_offsets[i].bParsed && m_offsets[i].cUsed == 'f' && m_offsets[i].lOffset )
 //      {
-//          m_vecObjects->AddFreeObject( PdfReference( static_cast<int>(m_offsets[i].lOffset), 1LL ) ); // TODO: do not hard code
+//          m_vecObjects->AddFreeObject( PdfReference( static_cast<int>(m_offsets[i].lOffset), PODOFO_LL_LITERAL(1) ) ); // TODO: do not hard code
 //      }
         else if( (!m_offsets[i].bParsed || m_offsets[i].cUsed == 'f') && i != 0 )
         {
-			m_vecObjects->AddFreeObject( PdfReference( static_cast<int>(i), 1LL ) ); // TODO: do not hard code generation number
+			m_vecObjects->AddFreeObject( PdfReference( static_cast<int>(i), PODOFO_LL_LITERAL(1) ) ); // TODO: do not hard code generation number
         }
     }
 
@@ -1093,7 +1179,7 @@ void PdfParser::ReadObjectFromStream( int nObjNo, int )
         if( m_offsets[i].bParsed && m_offsets[i].cUsed == 's' &&
 			m_offsets[i].lGeneration == nObjNo) 
         {
-            list.push_back(static_cast<long long>(i));
+            list.push_back(static_cast<pdf_int64>(i));
 		}
 	}
     
@@ -1242,7 +1328,7 @@ void PdfParser::CheckEOFMarker()
     const size_t nEOFTokenLen = 5;
     char pszBuff[nEOFTokenLen+1];
 
-    m_device.Device()->Seek(-nEOFTokenLen, std::ios_base::end );
+    m_device.Device()->Seek(-static_cast<int>(nEOFTokenLen), std::ios_base::end );
     if( IsStrictParsing() )
     {
         // For strict mode EOF marker must be at the very end of the file
@@ -1283,6 +1369,25 @@ void PdfParser::CheckEOFMarker()
     }
 }
 
+bool PdfParser::HasXRefStream()
+{
+   m_device.Device()->Tell();
+   m_device.Device()->Seek( m_nXRefOffset );
+   
+   if( !this->IsNextToken( "xref" ) )  {
+        //      if( m_ePdfVersion < ePdfVersion_1_5 )
+        //		Ulrich Arnold 19.10.2009, found linearized 1.3-pdf's with trailer-info in xref-stream
+       if( m_ePdfVersion < ePdfVersion_1_3 )  {
+           return false;
+       } else {
+           return true;
+       }
+   }
+
+   return false;
+}
+
+
 };
 
 
