@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfString.h"
@@ -52,14 +65,14 @@ static const char* genStrEscMap()
     const long lAllocLen = 256;
     char* map = static_cast<char*>(g_StrEscMap);
     memset( map, 0, sizeof(char) * lAllocLen );
-    map['\n'] = 'n'; // Line feed (LF)
-    map['\r'] = 'r'; // Carriage return (CR)
-    map['\t'] = 't'; // Horizontal tab (HT)
-    map['\b'] = 'b'; // Backspace (BS)
-    map['\f'] = 'f'; // Form feed (FF)
-    map[')'] = ')';  
-    map['('] = '(';  
-    map['\\'] = '\\';
+    map[static_cast<unsigned char>('\n')] = 'n'; // Line feed (LF)
+    map[static_cast<unsigned char>('\r')] = 'r'; // Carriage return (CR)
+    map[static_cast<unsigned char>('\t')] = 't'; // Horizontal tab (HT)
+    map[static_cast<unsigned char>('\b')] = 'b'; // Backspace (BS)
+    map[static_cast<unsigned char>('\f')] = 'f'; // Form feed (FF)
+    map[static_cast<unsigned char>(')')] = ')';  
+    map[static_cast<unsigned char>('(')] = '(';  
+    map[static_cast<unsigned char>('\\')] = '\\';
 
     return map;
 }
@@ -70,7 +83,7 @@ const char * const PdfString::m_escMap        = PdfStringNameSpace::genStrEscMap
 
 const PdfString PdfString::StringNull        = PdfString();
 #ifdef _MSC_VER
-const char  PdfString::s_pszUnicodeMarker[]  = { 0xFE, 0xFF };
+const char  PdfString::s_pszUnicodeMarker[]  = { static_cast<char>(0xFE), static_cast<char>(0xFF) };
 #else
 const char  PdfString::s_pszUnicodeMarker[]  = { static_cast<char>(0xFE), static_cast<char>(0xFF) };
 #endif
@@ -98,12 +111,24 @@ PdfString::PdfString( const char* pszString, const PdfEncoding * const pEncoding
 
 #if defined(_MSC_VER)  &&  _MSC_VER <= 1200			// nicht fr Visualstudio 6
 #else
-PdfString::PdfString( const wchar_t* pszString )
-    : m_bHex( false ), m_bUnicode( true ), m_pEncoding( NULL )
+PdfString::PdfString( const wchar_t* pszString, pdf_long lLen )
 {
+    setFromWchar_t(pszString, lLen);
+}
+#endif
+
+void PdfString::setFromWchar_t(const wchar_t* pszString, pdf_long lLen )
+{
+    m_bHex = false;
+    m_bUnicode = true;
+    m_pEncoding = NULL;
+
     if( pszString )
     {
-        pdf_long lLen = wcslen( pszString );
+        if (lLen == -1)
+        {
+            lLen = wcslen( pszString );
+        }
         if( sizeof(wchar_t) == 2 ) 
         {
             // We have UTF16
@@ -141,7 +166,6 @@ PdfString::PdfString( const wchar_t* pszString )
         }    
     }
 }
-#endif
 
 PdfString::PdfString( const char* pszString, pdf_long lLen, bool bHex, const PdfEncoding * const pEncoding )
     : m_bHex( bHex ), m_bUnicode( false ), m_pEncoding( pEncoding )
@@ -272,7 +296,7 @@ void PdfString::SetHexData( const char* pszHex, pdf_long lLen, PdfEncrypt* pEncr
     *pBuffer++ = '\0';
     *pBuffer++ = '\0';
 
-    // If the allocated internal buffer is to big (e.g. because of whitespaces in the data)
+    // If the allocated internal buffer is too big (e.g. because of whitespaces in the data)
     // copy to a smaller buffer so that PdfString::GetLength() will be correct
     lLen = pBuffer - m_buffer.GetBuffer();
     if( static_cast<size_t>(lLen) != m_buffer.GetSize() )
@@ -283,11 +307,21 @@ void PdfString::SetHexData( const char* pszHex, pdf_long lLen, PdfEncrypt* pEncr
     }
 
     if( pEncrypt )
-        pEncrypt->Encrypt( reinterpret_cast<unsigned char*>(m_buffer.GetBuffer()), 
-                           static_cast<unsigned int>(m_buffer.GetSize()-2) );
+    {
+        pdf_long outBufferLen = m_buffer.GetSize() - 2 - pEncrypt->CalculateStreamOffset();
+        PdfRefCountedBuffer outBuffer(outBufferLen + 16 - (outBufferLen % 16));
+        
+        pEncrypt->Decrypt( reinterpret_cast<unsigned char*>(m_buffer.GetBuffer()),
+                           static_cast<unsigned int>(m_buffer.GetSize()-2),
+                          reinterpret_cast<unsigned char*>(outBuffer.GetBuffer()),
+                          outBufferLen);
+        outBuffer.Resize(outBufferLen);
+        // Replace buffer with decrypted value
+        m_buffer = outBuffer;
+    }
 
     // Now check for the first two bytes, to see if we got a unicode string
-    if( m_buffer.GetSize()-2 > 2 ) 
+    if( m_buffer.GetSize() > 4 ) 
     {
 		m_bUnicode = (m_buffer.GetBuffer()[0] == static_cast<char>(0xFE) && m_buffer.GetBuffer()[1] == static_cast<char>(0xFF));
 		
@@ -308,45 +342,34 @@ void PdfString::Write ( PdfOutputDevice* pDevice, EPdfWriteMode eWriteMode, cons
     // Peter Petrov: 17 May 2008
     // Added check - m_buffer.GetSize()
     // Now we are not encrypting the empty strings (was access violation)!
-    if( pEncrypt && m_buffer.GetSize()) 
+    if( pEncrypt && m_buffer.GetSize())
     {
-        pdf_long nOffset = pEncrypt->CalculateStreamOffset();
-        pdf_long nLen = this->GetLength();
-        // Add two bytes for the s_pszUnicodeMarker
-        pdf_long nOutputLen = pEncrypt->CalculateStreamLength( (m_bUnicode ? nLen + 2 : nLen) );
+        pdf_long nInputBufferLen = m_buffer.GetSize();
+        pdf_long nUnicodeMarkerOffet = sizeof( PdfString::s_pszUnicodeMarker );
+        if( m_bUnicode )
+            nInputBufferLen += nUnicodeMarkerOffet;
         
-        char* pBuffer = new char [nOutputLen + 1];
-        memcpy(&pBuffer[nOffset], this->GetString(), this->GetLength());
+        char * pInputBuffer = new char[nInputBufferLen];
         
-        std::string enc = std::string(pBuffer, nOutputLen);
         if( m_bUnicode )
         {
-            std::string tmp( reinterpret_cast<const char*>(&PdfString::s_pszUnicodeMarker), 2 );
-
-            if( pEncrypt->GetEncryptAlgorithm() == PdfEncrypt::ePdfEncryptAlgorithm_AESV2 )
-            {
-                // Insert after the initial vector
-                enc.insert(16,tmp);
-                enc.erase(nOutputLen,std::string::npos);  
-            }
-            else
-            {
-                enc.insert(0,tmp);
-                enc.erase(nOutputLen,std::string::npos);  
-        }
-	
-            // Add two bytes for the s_pszUnicodeMarker
-            pEncrypt->Encrypt(enc, nLen + 2 );
+            memcpy(pInputBuffer, PdfString::s_pszUnicodeMarker, nUnicodeMarkerOffet);
+            memcpy(&pInputBuffer[nUnicodeMarkerOffet], m_buffer.GetBuffer(), m_buffer.GetSize());
         }
         else
-        {
-        pEncrypt->Encrypt(enc, nLen );
-        }
+            memcpy(pInputBuffer, m_buffer.GetBuffer(), m_buffer.GetSize());
+        
+        pdf_long nOutputBufferLen = pEncrypt->CalculateStreamLength(nInputBufferLen);
+        
+        char* pOutputBuffer = new char [nOutputBufferLen];
+        
+        pEncrypt->Encrypt(reinterpret_cast<const unsigned char*>(pInputBuffer), nInputBufferLen, reinterpret_cast<unsigned char*>(pOutputBuffer), nOutputBufferLen);
 
-        PdfString str( enc.c_str(), enc.length(), true );
+        PdfString str( pOutputBuffer, nOutputBufferLen, true );
         str.Write( pDevice, eWriteMode, NULL );
 
-        delete[] pBuffer;
+        delete[] pInputBuffer;
+        delete[] pOutputBuffer;
 		
         return;
     }
@@ -1043,6 +1066,11 @@ pdf_long PdfString::ConvertUTF16toUTF8( const pdf_utf16be* pszUtf16, pdf_long lL
     return target - pszUtf8;
 }
 
+PdfRefCountedBuffer &PdfString::GetBuffer(void)
+{
+	return m_buffer;
+}
+
 /* ---------------------------------------------------------------------
 
     Note A.
@@ -1064,5 +1092,3 @@ pdf_long PdfString::ConvertUTF16toUTF8( const pdf_utf16be* pszUtf16, pdf_long lL
 
 
 };
-
-
