@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfTokenizer.h"
@@ -88,14 +101,14 @@ const char* genEscMap()
     char* map = static_cast<char*>(g_EscMap);
     memset( map, 0, sizeof(char) * g_MapAllocLen );
 
-    map['n'] = '\n'; // Line feed (LF)
-    map['r'] = '\r'; // Carriage return (CR)
-    map['t'] = '\t'; // Horizontal tab (HT)
-    map['b'] = '\b'; // Backspace (BS)
-    map['f'] = '\f'; // Form feed (FF)
-    map[')'] = ')';  
-    map['('] = '(';  
-    map['\\'] = '\\';
+    map[static_cast<unsigned char>('n')] = '\n'; // Line feed (LF)
+    map[static_cast<unsigned char>('r')] = '\r'; // Carriage return (CR)
+    map[static_cast<unsigned char>('t')] = '\t'; // Horizontal tab (HT)
+    map[static_cast<unsigned char>('b')] = '\b'; // Backspace (BS)
+    map[static_cast<unsigned char>('f')] = '\f'; // Form feed (FF)
+    map[static_cast<unsigned char>(')')] = ')';  
+    map[static_cast<unsigned char>('(')] = '(';  
+    map[static_cast<unsigned char>('\\')] = '\\';
 
     return map;
 }
@@ -106,28 +119,28 @@ const char* genHexMap()
     char* map = static_cast<char*>(g_hexMap);
     memset( map, PdfTokenizer::HEX_NOT_FOUND, sizeof(char) * g_MapAllocLen );
     
-    map['0'] = 0x0;
-    map['1'] = 0x1;
-    map['2'] = 0x2;
-    map['3'] = 0x3;
-    map['4'] = 0x4;
-    map['5'] = 0x5;
-    map['6'] = 0x6;
-    map['7'] = 0x7;
-    map['8'] = 0x8;
-    map['9'] = 0x9;
-    map['a'] = 0xA;
-    map['b'] = 0xB;
-    map['c'] = 0xC;
-    map['d'] = 0xD;
-    map['e'] = 0xE;
-    map['f'] = 0xF;
-    map['A'] = 0xA;
-    map['B'] = 0xB;
-    map['C'] = 0xC;
-    map['D'] = 0xD;
-    map['E'] = 0xE;
-    map['F'] = 0xF;
+    map[static_cast<unsigned char>('0')] = 0x0;
+    map[static_cast<unsigned char>('1')] = 0x1;
+    map[static_cast<unsigned char>('2')] = 0x2;
+    map[static_cast<unsigned char>('3')] = 0x3;
+    map[static_cast<unsigned char>('4')] = 0x4;
+    map[static_cast<unsigned char>('5')] = 0x5;
+    map[static_cast<unsigned char>('6')] = 0x6;
+    map[static_cast<unsigned char>('7')] = 0x7;
+    map[static_cast<unsigned char>('8')] = 0x8;
+    map[static_cast<unsigned char>('9')] = 0x9;
+    map[static_cast<unsigned char>('a')] = 0xA;
+    map[static_cast<unsigned char>('b')] = 0xB;
+    map[static_cast<unsigned char>('c')] = 0xC;
+    map[static_cast<unsigned char>('d')] = 0xD;
+    map[static_cast<unsigned char>('e')] = 0xE;
+    map[static_cast<unsigned char>('f')] = 0xF;
+    map[static_cast<unsigned char>('A')] = 0xA;
+    map[static_cast<unsigned char>('B')] = 0xB;
+    map[static_cast<unsigned char>('C')] = 0xC;
+    map[static_cast<unsigned char>('D')] = 0xD;
+    map[static_cast<unsigned char>('E')] = 0xE;
+    map[static_cast<unsigned char>('F')] = 0xF;
     
     return map;
 }
@@ -194,7 +207,7 @@ PdfTokenizer::~PdfTokenizer()
 bool PdfTokenizer::GetNextToken( const char*& pszToken , EPdfTokenType* peType )
 {
     int  c; 
-    long long  counter  = 0;
+    pdf_int64  counter  = 0;
 
     // check first if there are quequed tokens and return them first
     if( m_deqQueque.size() )
@@ -219,7 +232,7 @@ bool PdfTokenizer::GetNextToken( const char*& pszToken , EPdfTokenType* peType )
         *peType = ePdfTokenType_Token;
 
     while( (c = m_device.Device()->Look()) != EOF
-           && counter < static_cast<long long>(m_buffer.GetSize()) )
+           && counter < static_cast<pdf_int64>(m_buffer.GetSize()) )
     {
         // ignore leading whitespaces
         if( !counter && IsWhitespace( c ) )
@@ -232,9 +245,16 @@ bool PdfTokenizer::GetNextToken( const char*& pszToken , EPdfTokenType* peType )
         else if( c == '%' ) 
         {
             // Consume all characters before the next line break
+			// 2011-04-19 Ulrich Arnold: accept 0x0D, 0x0A and oX0D 0x0A as one EOL
             do {
                 c = m_device.Device()->GetChar();
-            } while( c != EOF && c != 0x0A );
+            } while( c != EOF && c != 0x0D  && c != 0x0A );
+			
+            if ( c == 0x0D )
+			{
+                if ( m_device.Device()->Look() == 0x0A )
+	                c = m_device.Device()->GetChar();
+			}
             // If we've already read one or more chars of a token, return them, since
             // comments are treated as token-delimiting whitespace. Otherwise keep reading
             // at the start of the next line.
@@ -704,14 +724,29 @@ void PdfTokenizer::ReadString( PdfVariant& rVariant, PdfEncrypt* pEncrypt )
     if( bOctEscape )
         m_vecBuffer.push_back ( cOctValue );
 
-    if( pEncrypt && m_vecBuffer.size() )
-        pEncrypt->Encrypt( reinterpret_cast<unsigned char*>(&(m_vecBuffer[0])), 
-                           static_cast<unsigned int>(m_vecBuffer.size()) );
-
     if( m_vecBuffer.size() )
-        rVariant = PdfString( &(m_vecBuffer[0]), m_vecBuffer.size() );
+    {
+        if( pEncrypt )
+        {
+            pdf_long outLen = m_vecBuffer.size() - pEncrypt->CalculateStreamOffset();
+            char * outBuffer = new char[outLen + 16 - (outLen % 16)];
+            pEncrypt->Decrypt( reinterpret_cast<unsigned char*>(&(m_vecBuffer[0])),
+                              static_cast<unsigned int>(m_vecBuffer.size()),
+                              reinterpret_cast<unsigned char*>(outBuffer), outLen);
+            
+            rVariant = PdfString( outBuffer, outLen );
+            
+            delete[] outBuffer;
+        }
+        else
+        {
+            rVariant = PdfString( &(m_vecBuffer[0]), m_vecBuffer.size() );
+        }
+    }
     else
+    {
         rVariant = PdfString("");
+    }
 }
 
 void PdfTokenizer::ReadHexString( PdfVariant& rVariant, PdfEncrypt* pEncrypt )
