@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfWriter.h"
@@ -160,10 +173,23 @@ void PdfWriter::Write( PdfOutputDevice* pDevice )
         } catch( PdfError & e ) {
             // Make sure pXRef is always deleted
             delete pXRef;
+            
+            // P.Zent: Delete Encryption dictionary (cannot be reused)
+            if(m_pEncryptObj) {
+                m_vecObjects->RemoveObject(m_pEncryptObj->Reference());
+                delete m_pEncryptObj;
+            }
+            
             e.AddToCallstack( __FILE__, __LINE__ );
             throw e;
         }
     }
+    
+    // P.Zent: Delete Encryption dictionary (cannot be reused)
+    if(m_pEncryptObj) {
+        m_vecObjects->RemoveObject(m_pEncryptObj->Reference());
+        delete m_pEncryptObj;
+    }
 }
 
 void PdfWriter::WriteLinearized( PdfOutputDevice* /* pDevice */ )
@@ -595,7 +621,7 @@ void PdfWriter::CreateFileIdentifier( PdfString & identifier, const PdfObject* p
     pInfo->WriteObject( &device, m_eWriteMode, NULL );
 
     // calculate the MD5 Sum
-    identifier = PdfEncrypt::GetMD5String( reinterpret_cast<unsigned char*>(pBuffer),
+    identifier = PdfEncryptMD5Base::GetMD5String( reinterpret_cast<unsigned char*>(pBuffer),
                                            static_cast<unsigned int>(length.GetLength()) );
     free( pBuffer );
 
