@@ -1,22 +1,35 @@
 /***************************************************************************
-*   Copyright (C) 2006 by Dominik Seichter                                *
-*   domseichter@web.de                                                    *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU Library General Public License as       *
-*   published by the Free Software Foundation; either version 2 of the    *
-*   License, or (at your option) any later version.                       *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU Library General Public     *
-*   License along with this program; if not, write to the                 *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyright (C) 2006 by Dominik Seichter                                *
+ *   domseichter@web.de                                                    *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 2 of the    *
+ *   License, or (at your option) any later version.                       *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
+ *   Free Software Foundation, Inc.,                                       *
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
+ ***************************************************************************/
 
 #if defined(_MSC_VER)  &&  _MSC_VER <= 1200
 #pragma warning(disable: 4786)
@@ -38,6 +51,7 @@
 #include "base/PdfStream.h"
 #include "base/PdfVecObjects.h"
 
+#include "PdfAcroForm.h"
 #include "PdfDestination.h"
 #include "PdfFileSpec.h"
 #include "PdfFont.h"
@@ -55,23 +69,31 @@ using namespace std;
 
 namespace PoDoFo {
 
-PdfDocument::PdfDocument()
-    : m_fontCache( &m_vecObjects ), m_pOutlines( NULL ), 
-      m_pNamesTree( NULL ), m_pPagesTree( NULL ), 
-      m_pAcroForms( NULL )
+PdfDocument::PdfDocument(bool bEmpty)
+    : m_fontCache( &m_vecObjects ), 
+      m_pTrailer(NULL),
+      m_pCatalog(NULL),
+      m_pInfo(NULL),
+      m_pPagesTree( NULL ),
+      m_pAcroForms( NULL ),  
+      m_pOutlines( NULL ), 
+      m_pNamesTree( NULL )
 {
     m_vecObjects.SetParentDocument( this );
 
-    m_pTrailer = new PdfObject(); // The trailer is NO part of the vector of objects
-    m_pTrailer->SetOwner( &m_vecObjects );
-    m_pCatalog = m_vecObjects.CreateObject( "Catalog" );
+    if( !bEmpty ) 
+    {
+        m_pTrailer = new PdfObject(); // The trailer is NO part of the vector of objects
+        m_pTrailer->SetOwner( &m_vecObjects );
+        m_pCatalog = m_vecObjects.CreateObject( "Catalog" );
+        
+        m_pInfo = new PdfInfo( &m_vecObjects );
+        
+        m_pTrailer->GetDictionary().AddKey( "Root", m_pCatalog->Reference() );
+        m_pTrailer->GetDictionary().AddKey( "Info", m_pInfo->GetObject()->Reference() );
 
-    m_pInfo = new PdfInfo( &m_vecObjects );
-
-    m_pTrailer->GetDictionary().AddKey( "Root", m_pCatalog->Reference() );
-    m_pTrailer->GetDictionary().AddKey( "Info", m_pInfo->GetObject()->Reference() );
-
-    InitPagesTree();
+        InitPagesTree();
+    }
 }
 
 PdfDocument::~PdfDocument()
@@ -167,53 +189,64 @@ PdfPage* PdfDocument::GetPage( int nIndex ) const
     return m_pPagesTree->GetPage( nIndex );
 }
 
-PdfFont* PdfDocument::CreateFont( const char* pszFontName,                                   
+PdfFont* PdfDocument::CreateFont( const char* pszFontName,
+                                  bool bSymbolCharset,
                                   const PdfEncoding * const pEncoding, 
                                   PdfFontCache::EFontCreationFlags eFontCreationFlags, 
                                   bool bEmbedd )
 {
-    return m_fontCache.GetFont( pszFontName, false, false, bEmbedd, eFontCreationFlags, pEncoding );
+    return m_fontCache.GetFont( pszFontName, false, false, bSymbolCharset, bEmbedd, eFontCreationFlags, pEncoding );
 }
 
-PdfFont* PdfDocument::CreateFont( const char* pszFontName, bool bBold, bool bItalic, 
+PdfFont* PdfDocument::CreateFont( const char* pszFontName, bool bBold, bool bItalic, bool bSymbolCharset,
                                   const PdfEncoding * const pEncoding, 
                                   PdfFontCache::EFontCreationFlags eFontCreationFlags,
                                   bool bEmbedd, const char* pszFileName )
 {
-    return m_fontCache.GetFont( pszFontName, bBold, bItalic, bEmbedd, eFontCreationFlags, pEncoding, pszFileName );
+    return m_fontCache.GetFont( pszFontName, bBold, bItalic, bSymbolCharset, bEmbedd, eFontCreationFlags, pEncoding, pszFileName );
 }
 
-#ifdef _WIN32
-PdfFont* PdfDocument::CreateFont( const wchar_t* pszFontName, const PdfEncoding * const pEncoding, 
+#if defined(_WIN32) && !defined(PODOFO_NO_FONTMANAGER)
+PdfFont* PdfDocument::CreateFont( const wchar_t* pszFontName, bool bSymbolCharset, const PdfEncoding * const pEncoding, 
                                   bool bEmbedd )
 {
-    return m_fontCache.GetFont( pszFontName, false, false, bEmbedd, pEncoding );
+    return m_fontCache.GetFont( pszFontName, false, false, bSymbolCharset, bEmbedd, pEncoding );
 }
 
-PdfFont* PdfDocument::CreateFont( const wchar_t* pszFontName, bool bBold, bool bItalic, 
+PdfFont* PdfDocument::CreateFont( const wchar_t* pszFontName, bool bBold, bool bItalic, bool bSymbolCharset,
                                   const PdfEncoding * const pEncoding, bool bEmbedd )
 {
-    return m_fontCache.GetFont( pszFontName, bBold, bItalic, bEmbedd, pEncoding );
+    return m_fontCache.GetFont( pszFontName, bBold, bItalic, bSymbolCharset, bEmbedd, pEncoding );
+}
+
+PdfFont* PdfDocument::CreateFont( const LOGFONTA &logFont, const PdfEncoding * const pEncoding, bool bEmbedd )
+{
+    return m_fontCache.GetFont( logFont, bEmbedd, pEncoding );
+}
+
+PdfFont* PdfDocument::CreateFont( const LOGFONTW &logFont, const PdfEncoding * const pEncoding, bool bEmbedd )
+{
+    return m_fontCache.GetFont( logFont, bEmbedd, pEncoding );
 }
 #endif // _WIN32
 
-PdfFont* PdfDocument::CreateFontSubset( const char* pszFontName, bool bBold, bool bItalic, 
-					                    const PdfEncoding * const pEncoding, const char* pszFileName )
+PdfFont* PdfDocument::CreateFontSubset( const char* pszFontName, bool bBold, bool bItalic, bool bSymbolCharset,
+                                        const PdfEncoding * const pEncoding, const char* pszFileName )
 {
-    return m_fontCache.GetFontSubset( pszFontName, bBold, bItalic, pEncoding, pszFileName );
+    return m_fontCache.GetFontSubset( pszFontName, bBold, bItalic, bSymbolCharset, pEncoding, pszFileName );
 }
 
-#ifdef _WIN32
-PdfFont* PdfDocument::CreateFontSubset( const wchar_t* pszFontName, bool bBold, bool bItalic, 
-					                    const PdfEncoding * const pEncoding)
+#if defined(_WIN32) && !defined(PODOFO_NO_FONTMANAGER)
+PdfFont* PdfDocument::CreateFontSubset( const wchar_t* pszFontName, bool bBold, bool bItalic, bool bSymbolCharset,
+                                        const PdfEncoding * const pEncoding)
 {
     PODOFO_RAISE_ERROR_INFO( ePdfError_Unknown, "Subsets are not yet implemented for unicode on windows." );
 }
 #endif // _WIN32
 
-PdfFont* PdfDocument::CreateFont( FT_Face face, const PdfEncoding * const pEncoding, bool bEmbedd )
+PdfFont* PdfDocument::CreateFont( FT_Face face, bool bSymbolCharset, const PdfEncoding * const pEncoding, bool bEmbedd )
 {
-    return m_fontCache.GetFont( face, bEmbedd, pEncoding );
+    return m_fontCache.GetFont( face, bSymbolCharset, bEmbedd, pEncoding );
 }
 
 PdfFont* PdfDocument::CreateDuplicateFontType1( PdfFont * pFont, const char * pszSuffix )
@@ -226,6 +259,16 @@ PdfPage* PdfDocument::CreatePage( const PdfRect & rSize )
     return m_pPagesTree->CreatePage( rSize );
 }
 
+void PdfDocument::CreatePages( const std::vector<PdfRect>& vecSizes )
+{
+    m_pPagesTree->CreatePages( vecSizes );
+}
+
+PdfPage* PdfDocument::InsertPage( const PdfRect & rSize, int atIndex)
+{
+	return m_pPagesTree->InsertPage( rSize, atIndex );
+}
+
 void PdfDocument::EmbedSubsetFonts()
 {
 	m_fontCache.EmbedSubsetFonts();
@@ -323,33 +366,138 @@ const PdfDocument & PdfDocument::Append( const PdfMemDocument & rDoc, bool bAppe
     return *this;
 }
 
+const PdfDocument &PdfDocument::InsertExistingPageAt( const PdfMemDocument & rDoc, int nPageIndex, int nAtIndex)
+{
+	/* copy of PdfDocument::Append, only restricts which page to add */
+    unsigned int difference = static_cast<unsigned int>(m_vecObjects.GetSize() + m_vecObjects.GetFreeObjects().size());
+
+
+    // Ulrich Arnold 30.7.2009: Because GetNextObject uses m_nObjectCount instead 
+    //                          of m_vecObjects.GetSize()+m_vecObjects.GetFreeObjects().size()+1
+    //                          make sure the free objects are already present before appending to
+	//                          prevent overlapping obj-numbers
+
+    // create all free objects again, to have a clean free object list
+    TCIPdfReferenceList itFree = rDoc.GetObjects().GetFreeObjects().begin();
+    while( itFree != rDoc.GetObjects().GetFreeObjects().end() )
+    {
+        m_vecObjects.AddFreeObject( PdfReference( (*itFree).ObjectNumber() + difference, (*itFree).GenerationNumber() ) );
+
+        ++itFree;
+    }
+
+	// append all objects first and fix their references
+    TCIVecObjects it           = rDoc.GetObjects().begin();
+    while( it != rDoc.GetObjects().end() )
+    {
+        PdfObject* pObj = new PdfObject( PdfReference( 
+                                             static_cast<unsigned int>((*it)->Reference().ObjectNumber() + difference), (*it)->Reference().GenerationNumber() ), *(*it) );
+        m_vecObjects.push_back( pObj );
+
+        if( (*it)->IsDictionary() && (*it)->HasStream() )
+            *(pObj->GetStream()) = *((*it)->GetStream());
+
+        PdfError::LogMessage( eLogSeverity_Information,
+                              "Fixing references in %i %i R by %i\n", pObj->Reference().ObjectNumber(), pObj->Reference().GenerationNumber(), difference );
+        FixObjectReferences( pObj, difference );
+
+        ++it;
+    }
+
+    const PdfName inheritableAttributes[] = {
+        PdfName("Resources"),
+        PdfName("MediaBox"),
+        PdfName("CropBox"),
+        PdfName("Rotate"),
+        PdfName::KeyNull
+    };
+
+    // append all pages now to our page tree
+    for(int i=0;i<rDoc.GetPageCount();i++ )
+    {
+        if (i != nPageIndex) {
+            continue;
+        }
+
+        PdfPage*      pPage = rDoc.GetPage( i );
+        PdfObject*    pObj  = m_vecObjects.GetObject( PdfReference( pPage->GetObject()->Reference().ObjectNumber() + difference, pPage->GetObject()->Reference().GenerationNumber() ) );
+        if( pObj->IsDictionary() && pObj->GetDictionary().HasKey( "Parent" ) )
+            pObj->GetDictionary().RemoveKey( "Parent" );
+
+        // Deal with inherited attributes
+        const PdfName* pInherited = inheritableAttributes;
+        while( pInherited->GetLength() != 0 ) 
+        {
+	    const PdfObject* pAttribute = pPage->GetInheritedKey( *pInherited ); 
+	    if( pAttribute )
+	    {
+	        PdfObject attribute( *pAttribute );
+	        FixObjectReferences( &attribute, difference );
+	        pObj->GetDictionary().AddKey( *pInherited, attribute );
+	    }
+
+	    ++pInherited;
+        }
+
+        m_pPagesTree->InsertPage( nAtIndex <= 0 ? ePdfPageInsertionPoint_InsertBeforeFirstPage : nAtIndex - 1, pObj );
+    }
+
+    // append all outlines
+    PdfOutlineItem* pRoot       = this->GetOutlines();
+    PdfOutlines*    pAppendRoot = const_cast<PdfMemDocument&>(rDoc).GetOutlines( PoDoFo::ePdfDontCreateObject );
+    if( pAppendRoot && pAppendRoot->First() ) 
+    {
+        // only append outlines if appended document has outlines
+        while( pRoot && pRoot->Next() ) 
+	    pRoot = pRoot->Next();
+    
+        PdfReference ref( pAppendRoot->First()->GetObject()->Reference().ObjectNumber() + difference, pAppendRoot->First()->GetObject()->Reference().GenerationNumber() );
+        pRoot->InsertChild( new PdfOutlines( m_vecObjects.GetObject( ref ) ) );
+    }
+    
+    // TODO: merge name trees
+    // ToDictionary -> then iteratate over all keys and add them to the new one
+    return *this;
+}
+
 PdfRect PdfDocument::FillXObjectFromDocumentPage( PdfXObject * pXObj, const PdfMemDocument & rDoc, int nPage, bool bUseTrimBox )
 {
     unsigned int difference = static_cast<unsigned int>(m_vecObjects.GetSize() + m_vecObjects.GetFreeObjects().size());
+    Append( rDoc, false );
+    PdfPage* pPage = rDoc.GetPage( nPage );
 
-	Append( rDoc, false );
+    return FillXObjectFromPage( pXObj, pPage, bUseTrimBox, difference );
+}
 
+PdfRect PdfDocument::FillXObjectFromExistingPage( PdfXObject * pXObj, int nPage, bool bUseTrimBox )
+{
+    PdfPage* pPage = GetPage( nPage );
+
+    return FillXObjectFromPage( pXObj, pPage, bUseTrimBox, 0 );
+}
+
+PdfRect PdfDocument::FillXObjectFromPage( PdfXObject * pXObj, const PdfPage * pPage, bool bUseTrimBox, unsigned int difference )
+{
     // TODO: remove unused objects: page, ...
 
-    PdfPage*      pPage = rDoc.GetPage( nPage );
     PdfObject*    pObj  = m_vecObjects.GetObject( PdfReference( pPage->GetObject()->Reference().ObjectNumber() + difference, pPage->GetObject()->Reference().GenerationNumber() ) );
-    PdfRect		  box  = pPage->GetMediaBox();
+    PdfRect       box  = pPage->GetMediaBox();
 
-	// intersect with crop-box
-	box.Intersect( pPage->GetCropBox() );
+    // intersect with crop-box
+    box.Intersect( pPage->GetCropBox() );
 
-	// intersect with trim-box according to parameter
-	if ( bUseTrimBox )
-		box.Intersect( pPage->GetTrimBox() );
+    // intersect with trim-box according to parameter
+    if ( bUseTrimBox )
+        box.Intersect( pPage->GetTrimBox() );
 
-	// link resources from external doc to x-object
+    // link resources from external doc to x-object
     if( pObj->IsDictionary() && pObj->GetDictionary().HasKey( "Resources" ) )
         pXObj->GetContentsForAppending()->GetDictionary().AddKey( "Resources" , pObj->GetDictionary().GetKey( "Resources" ) );
 
-	// copy top-level content from external doc to x-object
+    // copy top-level content from external doc to x-object
     if( pObj->IsDictionary() && pObj->GetDictionary().HasKey( "Contents" ) )
     {
-		// get direct pointer to contents
+        // get direct pointer to contents
         PdfObject* pContents;
         if( pObj->GetDictionary().GetKey( "Contents" )->IsReference() )
             pContents = m_vecObjects.GetObject( pObj->GetDictionary().GetKey( "Contents" )->GetReference() );
@@ -358,46 +506,62 @@ PdfRect PdfDocument::FillXObjectFromDocumentPage( PdfXObject * pXObj, const PdfM
 
         if( pContents->IsArray() )
         {
-			// copy array as one stream to xobject
+            // copy array as one stream to xobject
             PdfArray pArray = pContents->GetArray();
 
             PdfObject*  pObj = pXObj->GetContentsForAppending();
             PdfStream*  pObjStream = pObj->GetStream();
 
             TVecFilters vFilters;
-		    vFilters.push_back( ePdfFilter_FlateDecode );
+            vFilters.push_back( ePdfFilter_FlateDecode );
             pObjStream->BeginAppend( vFilters );
 
-			TIVariantList it;
+            TIVariantList it;
             for(it = pArray.begin(); it != pArray.end(); it++)
             {
-				if ( it->IsReference() )
-				{
-					// TODO: not very efficient !!
-					PdfObject*  pObj = GetObjects()->GetObject( it->GetReference() );
+                if ( it->IsReference() )
+                {
+                    // TODO: not very efficient !!
+                    PdfObject*  pObj = GetObjects()->GetObject( it->GetReference() );
 
-		            PdfStream*  pcontStream = pObj->GetStream();
+                    while (pObj!=NULL)
+                    {
+                        if (pObj->IsReference())    // Recursively look for the stream
+                        {
+                            pObj = GetObjects()->GetObject( pObj->GetReference() );
+                        }
+                        else if (pObj->HasStream())
+                        {
+                            PdfStream*  pcontStream = pObj->GetStream();
 
-		            char*       pcontStreamBuffer;
-			        pdf_long    pcontStreamLength;
-		            pcontStream->GetFilteredCopy( &pcontStreamBuffer, &pcontStreamLength );
-		    
-					pObjStream->Append( pcontStreamBuffer, pcontStreamLength );
-					free( pcontStreamBuffer );
-				}
-				else
-				{
-					string str;
-	                it->ToString( str );
-	                pObjStream->Append( str );
-	                pObjStream->Append( " " );
-				}
-			}
+                            char*       pcontStreamBuffer;
+                            pdf_long    pcontStreamLength;
+                            pcontStream->GetFilteredCopy( &pcontStreamBuffer, &pcontStreamLength );
+    
+                            pObjStream->Append( pcontStreamBuffer, pcontStreamLength );
+                            free( pcontStreamBuffer );
+                            break;
+                        }
+                        else
+                        {
+                            throw ePdfError_InvalidStream;
+                            break;
+                        }
+                    }
+                }
+                else
+                {
+                    string str;
+                    it->ToString( str );
+                    pObjStream->Append( str );
+                    pObjStream->Append( " " );
+                }
+            }
             pObjStream->EndAppend();
         }
-		else if( pContents->HasStream() )
+        else if( pContents->HasStream() )
         {
-			// copy stream to xobject
+            // copy stream to xobject
             PdfObject*  pObj = pXObj->GetContentsForAppending();
             PdfStream*  pObjStream = pObj->GetStream();
             PdfStream*  pcontStream = pContents->GetStream();
@@ -405,16 +569,16 @@ PdfRect PdfDocument::FillXObjectFromDocumentPage( PdfXObject * pXObj, const PdfM
             pdf_long    pcontStreamLength;
 
             TVecFilters vFilters;
-		    vFilters.push_back( ePdfFilter_FlateDecode );
+            vFilters.push_back( ePdfFilter_FlateDecode );
             pObjStream->BeginAppend( vFilters );
             pcontStream->GetFilteredCopy( &pcontStreamBuffer, &pcontStreamLength );
             pObjStream->Append( pcontStreamBuffer, pcontStreamLength );
             free( pcontStreamBuffer );
             pObjStream->EndAppend();
         }
-		else
+        else
         {
-	        PODOFO_RAISE_ERROR( ePdfError_InternalLogic );
+            PODOFO_RAISE_ERROR( ePdfError_InternalLogic );
         }
     }
 
@@ -698,7 +862,7 @@ PdfNamesTree* PdfDocument::GetNamesTree( bool bCreate )
     return m_pNamesTree;
 }
 
-PdfAcroForm* PdfDocument::GetAcroForm( bool bCreate, PdfAcroForm::EPdfAcroFormDefaulAppearance eDefaultAppearance )
+PdfAcroForm* PdfDocument::GetAcroForm( bool bCreate, EPdfAcroFormDefaulAppearance eDefaultAppearance )
 {
     PdfObject* pObj;
 
@@ -735,7 +899,26 @@ void PdfDocument::AttachFile( const PdfFileSpec & rFileSpec )
         PODOFO_RAISE_ERROR( ePdfError_InvalidHandle );
     }
 
-    pNames->AddValue( "EmbeddedFiles", rFileSpec.GetFilename(), rFileSpec.GetObject()->Reference() );
+    pNames->AddValue( "EmbeddedFiles", rFileSpec.GetFilename(false), rFileSpec.GetObject()->Reference() );
+}
+    
+PdfFileSpec* PdfDocument::GetAttachment( const PdfString & rName )
+{
+    PdfNamesTree* pNames = this->GetNamesTree();
+    
+    if( !pNames )
+    {
+        return NULL;
+    }
+    
+    PdfObject* pObj = pNames->GetValue( "EmbeddedFiles", rName );
+    
+    if( !pObj )
+    {
+        return NULL;
+    }
+    
+    return new PdfFileSpec(pObj);
 }
 
 void PdfDocument::SetInfo( PdfInfo* pInfo )
