@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #ifndef _PDF_DOCUMENT_H_
@@ -37,6 +50,7 @@ class PdfDestination;
 class PdfDictionary;
 class PdfFileSpec;
 class PdfFont;
+class PdfFontConfigWrapper;
 class PdfInfo;
 class PdfMemDocument;
 class PdfNamesTree;
@@ -120,7 +134,7 @@ class PODOFO_DOC_API PdfDocument {
      *  \returns PdfObject the AcroForm dictionary
      */
     PdfAcroForm* GetAcroForm( bool bCreate = ePdfCreateObject,
-                              PdfAcroForm::EPdfAcroFormDefaulAppearance eDefaultAppearance = PdfAcroForm::ePdfAcroFormDefaultAppearance_BlackText12pt);
+                              EPdfAcroFormDefaulAppearance eDefaultAppearance = ePdfAcroFormDefaultAppearance_BlackText12pt);
 
     /** Get access to the pages tree.
      *  Better use GetPage and CreatePage methods.
@@ -145,6 +159,7 @@ class PODOFO_DOC_API PdfDocument {
 
     /** Creates a PdfFont object
      *  \param pszFontName name of the font as it is known to the system
+	  *  \param bSymbolCharset whether to use symbol charset, rather than unicode charset
      *  \param pEncoding the encoding of the font. The font will not take ownership of this object.     
      *  \param eFontCreationFlags special flag to specify how fonts should be created
      *  \param bEmbedd specifies whether this font should be embedded in the PDF file.
@@ -153,7 +168,7 @@ class PODOFO_DOC_API PdfDocument {
      *  \returns PdfFont* a pointer to a new PdfFont object.
      *           The returned object is owned by the PdfDocument.
      */
-    PdfFont* CreateFont( const char* pszFontName, 
+    PdfFont* CreateFont( const char* pszFontName, bool bSymbolCharset = false, 
                          const PdfEncoding * const pEncoding = PdfEncodingFactory::GlobalWinAnsiEncodingInstance(), 
                          PdfFontCache::EFontCreationFlags eFontCreationFlags = PdfFontCache::eFontCreationFlags_AutoSelectBase14,
                          bool bEmbedd = true );
@@ -162,6 +177,7 @@ class PODOFO_DOC_API PdfDocument {
      *  \param pszFontName name of the font as it is known to the system
      *  \param bBold if true search for a bold font
      *  \param bItalic if true search for an italic font
+	  *  \param bSymbolCharset whether to use symbol charset, rather than unicode charset
      *  \param pEncoding the encoding of the font. The font will not take ownership of this object.     
      *  \param eFontCreationFlags special flag to specify how fonts should be created
      *  \param bEmbedd specifies whether this font should be embedded in the PDF file.
@@ -170,7 +186,7 @@ class PODOFO_DOC_API PdfDocument {
      *
      *  \returns PdfFont* a pointer to a new PdfFont object.
      */
-    PdfFont* CreateFont( const char* pszFontName, bool bBold, bool bItalic, 
+    PdfFont* CreateFont( const char* pszFontName, bool bBold, bool bItalic, bool bSymbolCharset = false,
                          const PdfEncoding * const pEncoding = PdfEncodingFactory::GlobalWinAnsiEncodingInstance(), 
                          PdfFontCache::EFontCreationFlags eFontCreationFlags = PdfFontCache::eFontCreationFlags_AutoSelectBase14,
                          bool bEmbedd = true, const char* pszFileName = NULL );
@@ -178,6 +194,7 @@ class PODOFO_DOC_API PdfDocument {
 #ifdef _WIN32
     /** Creates a PdfFont object
      *  \param pszFontName name of the font as it is known to the system
+	  *  \param bSymbolCharset whether to use symbol charset, rather than unicode charset
      *  \param pEncoding the encoding of the font. The font will not take ownership of this object.     
      *  \param bEmbedd specifies whether this font should be embedded in the PDF file.
      *         Embedding fonts is usually a good idea.
@@ -189,13 +206,14 @@ class PODOFO_DOC_API PdfDocument {
      *  with unicode characters. On Unix systes you can also path
      *  UTF-8 to the const char* overload.
      */
-    PdfFont* CreateFont( const wchar_t* pszFontName, const PdfEncoding * const pEncoding = PdfEncodingFactory::GlobalWinAnsiEncodingInstance(), 
+    PdfFont* CreateFont( const wchar_t* pszFontName, bool bSymbolCharset = false, const PdfEncoding * const pEncoding = PdfEncodingFactory::GlobalWinAnsiEncodingInstance(), 
                          bool bEmbedd = true );
 
     /** Creates a PdfFont object
      *  \param pszFontName name of the font as it is known to the system
      *  \param bBold if true search for a bold font
      *  \param bItalic if true search for an italic font
+	  *  \param bSymbolCharset whether to use symbol charset, rather than unicode charset
      *  \param pEncoding the encoding of the font. The font will not take ownership of this object.     
      *  \param bEmbedd specifies whether this font should be embedded in the PDF file.
      *         Embedding fonts is usually a good idea.
@@ -207,20 +225,28 @@ class PODOFO_DOC_API PdfDocument {
      *  with unicode characters. On Unix systes you can also path
      *  UTF-8 to the const char* overload.
      */
-    PdfFont* CreateFont( const wchar_t* pszFontName, bool bBold, bool bItalic, 
+    PdfFont* CreateFont( const wchar_t* pszFontName, bool bBold, bool bItalic, bool bSymbolCharset = false,
                          const PdfEncoding * const pEncoding = PdfEncodingFactory::GlobalWinAnsiEncodingInstance(), 
                          bool bEmbedd = true);
+
+	 PdfFont* CreateFont( const LOGFONTA &logFont, const PdfEncoding * const pEncoding = PdfEncodingFactory::GlobalWinAnsiEncodingInstance(),
+								 bool bEmbedd = true );
+
+	 PdfFont* CreateFont( const LOGFONTW &logFont, const PdfEncoding * const pEncoding = PdfEncodingFactory::GlobalWinAnsiEncodingInstance(),
+								 bool bEmbedd = true );
+
 #endif // _WIN32
 
     /** Creates a PdfFont object
      *  \param face a valid freetype font handle (will be free'd by PoDoFo)
+	  *  \param bSymbolCharset whether to use symbol charset, rather than unicode charset
      *  \param pEncoding the encoding of the font. The font will not take ownership of this object.     
      *  \param bEmbedd specifies whether this font should be embedded in the PDF file.
      *         Embedding fonts is usually a good idea.
      *  \returns PdfFont* a pointer to a new PdfFont object.
      *           The returned object is owned by the PdfDocument.
      */
-    PdfFont* CreateFont( FT_Face face, const PdfEncoding * const pEncoding = PdfEncodingFactory::GlobalWinAnsiEncodingInstance(), bool bEmbedd = true );
+    PdfFont* CreateFont( FT_Face face, bool bSymbolCharset = false, const PdfEncoding * const pEncoding = PdfEncodingFactory::GlobalWinAnsiEncodingInstance(), bool bEmbedd = true );
 
     /** Creates a duplicate Type1-PdfFont with a new Id
      *  \param pFont is the existing font 
@@ -230,7 +256,7 @@ class PODOFO_DOC_API PdfDocument {
      *  TODO: DS: Make this generic so that it will work 
      *            for any font type!
      */
-	PdfFont* CreateDuplicateFontType1( PdfFont * pFont, const char * pszSuffix );
+    PdfFont* CreateDuplicateFontType1( PdfFont * pFont, const char * pszSuffix );
 
 	/** Creates a font subset which contains only a few characters and is embedded.
      *
@@ -239,12 +265,13 @@ class PODOFO_DOC_API PdfDocument {
      *  \param pszFontName name of the font as it is known to the system
      *  \param bBold if true search for a bold font
      *  \param bItalic if true search for an italic font
+	  *  \param bSymbolCharset whether to use symbol charset, rather than unicode charset
      *  \param pEncoding the encoding of the font. The font will not take ownership of this object.     
      *  \param pszFileName optional path of a fontfile which should be used
      *
      *  \returns PdfFont* a pointer to a new PdfFont object.
      */
-    PdfFont* CreateFontSubset( const char* pszFontName, bool bBold, bool bItalic, 
+    PdfFont* CreateFontSubset( const char* pszFontName, bool bBold, bool bItalic, bool bSymbolCharset = false,
 			       const PdfEncoding * const pEncoding = PdfEncodingFactory::GlobalWinAnsiEncodingInstance(),
 			       const char* pszFileName = NULL);
 
@@ -256,15 +283,16 @@ class PODOFO_DOC_API PdfDocument {
      *  \param pszFontName name of the font as it is known to the system
      *  \param bBold if true search for a bold font
      *  \param bItalic if true search for an italic font
+	  *  \param bSymbolCharset whether to use symbol charset, rather than unicode charset
      *  \param pEncoding the encoding of the font. The font will not take ownership of this object.     
      *
      *  \returns PdfFont* a pointer to a new PdfFont object.
- 	 *
+     *
      *  This is an overloaded member function to allow working
      *  with unicode characters. On Unix systes you can also path
      *  UTF-8 to the const char* overload.
      */
-    PdfFont* CreateFontSubset( const wchar_t* pszFontName, bool bBold, bool bItalic, 
+    PdfFont* CreateFontSubset( const wchar_t* pszFontName, bool bBold, bool bItalic, bool bSymbolCharset = false,
 			       const PdfEncoding * const = PdfEncodingFactory::GlobalWinAnsiEncodingInstance() );
 #endif // _WIN32
 
@@ -276,10 +304,10 @@ class PODOFO_DOC_API PdfDocument {
     inline FT_Library GetFontLibrary() const;
 	
     /** Embeds all pending subset-fonts, is automatically done on Write().
-	 *  Just call explicit in case PdfDocument is needed as XObject
+     *  Just call explicit in case PdfDocument is needed as XObject
      *
      */
-	void EmbedSubsetFonts();
+    void EmbedSubsetFonts();
 
     /** Creates a new page object and inserts it into the internal
      *  page tree. 
@@ -291,6 +319,26 @@ class PODOFO_DOC_API PdfDocument {
      */
     PdfPage* CreatePage( const PdfRect & rSize );
 
+
+    /** Creates several new page objects and inserts them into the internal
+     *  page tree. 
+     *  The created pages are owned by the PdfDocument
+     *  and will get deleted along with it!
+     *
+     *  \param vecSizes a vector PdfRect's specifying the size of the pages (i.e the /MediaBox key) in PDF Units
+     */
+    void CreatePages( const std::vector<PdfRect>& vecSizes );
+
+    /** Creates a new page object and inserts it at index atIndex.
+     *  The returned page is owned by the pages tree and will get deleted along
+     *  with it!
+     *
+     *  \param rSize a PdfRect specifying the size of the page (i.e the /MediaBox key) in PDF units
+     *  \param atIndex index where to insert the new page (0-based)
+     *  \returns a pointer to a PdfPage object
+     */
+    PdfPage* InsertPage( const PdfRect & rSize, int atIndex);
+
     /** Appends another PdfDocument to this document
      *  \param rDoc the document to append
      *  \param bAppendAll specifies whether pages and outlines are appended too
@@ -298,20 +346,54 @@ class PODOFO_DOC_API PdfDocument {
      */
     const PdfDocument & Append( const PdfMemDocument & rDoc, bool bAppendAll = true  );
 
+    /** Inserts existing page from another PdfMemDocument to this document
+     *  \param rDoc the document to append from
+     *  \param nPageIndex Page index to append (0-based), from rDoc
+     *  \param nAtIndex Index at which add the page in this document
+     *  \returns this document
+     */
+    const PdfDocument &InsertExistingPageAt( const PdfMemDocument & rDoc, int nPageIndex, int nAtIndex);
+
     /** Fill an existing empty XObject from a page of another document
+     *  This will append the other document with this one
      *  \param pXObj pointer to the XOject
      *  \param rDoc the document to embedd into XObject
      *  \param nPage page-number to embedd into XObject
-	 *	\param bUseTrimBox if true try to use trimbox for size of xobject
+     *  \param bUseTrimBox if true try to use trimbox for size of xobject
      *  \returns the bounding box
      */
     PdfRect FillXObjectFromDocumentPage( PdfXObject * pXObj, const PdfMemDocument & rDoc, int nPage, bool bUseTrimBox );
 
+    /** Fill an existing empty XObject from an existing page from the current document
+     *  If you need a page from another document use FillXObjectFromDocumentPage, or append the documents manually
+     *  \param pXObj pointer to the XOject
+     *  \param nPage page-number to embedd into XObject
+     *  \param bUseTrimBox if true try to use trimbox for size of xobject
+     *  \returns the bounding box
+     */
+    PdfRect FillXObjectFromExistingPage( PdfXObject * pXObj, int nPage, bool bUseTrimBox );
+
+    /** Fill an existing empty XObject from an existing page pointer from the current document
+     *  This is the implementation for FillXObjectFromDocumentPage and FillXObjectFromExistingPage, you should use those directly instead of this
+     *  \param pXObj pointer to the XOject
+     *  \param pPage pointer to the page to embedd into XObject
+     *  \param bUseTrimBox if true try to use trimbox for size of xobject
+     *  \returns the bounding box
+     */
+    PdfRect FillXObjectFromPage( PdfXObject * pXObj, const PdfPage * pPage, bool bUseTrimBox, unsigned int difference );
+
     /** Attach a file to the document.
      *  \param rFileSpec a file specification
      */
     void AttachFile( const PdfFileSpec & rFileSpec );
 
+    /** Get an attached file's filespec
+     *  \param rName the name of the attachment
+     *  \return the file specification object if the file exists, NULL otherwise
+     *          The file specification object is not owned by the document and must be deleted by the caller
+     */
+    PdfFileSpec* GetAttachment( const PdfString & rName );
+    
     /** Adds a PdfDestination into the global Names tree
      *  with the specified name, optionally replacing one of the same name
      *  \param rDest the destination to be assigned
@@ -482,10 +564,20 @@ class PODOFO_DOC_API PdfDocument {
      */
     inline const PdfVecObjects* GetObjects() const;
 
+    /**
+     * Set wrapper for the fontconfig library.
+     * Useful to avoid initializing Fontconfig multiple times.
+     *
+     * This setter can be called until first use of Fontconfig
+     * as the library is initialized at first use.
+     */
+    inline void SetFontConfigWrapper(const PdfFontConfigWrapper & rFontConfig);
+
  protected:
     /** Construct a new (empty) PdfDocument
+     *  \param bEmtpy if true NO default objects (such as catalog) are created.
      */
-    PdfDocument();
+    PdfDocument( bool bEmpty = false );
 
     /** Set the info object containing meta information.
      *  Deletes any old info object.
@@ -577,6 +669,12 @@ class PODOFO_DOC_API PdfDocument {
 
  protected:
     PdfFontCache    m_fontCache;
+    PdfObject*      m_pTrailer;
+    PdfObject*      m_pCatalog;
+
+    PdfInfo*        m_pInfo;
+    PdfPagesTree*   m_pPagesTree;
+    PdfAcroForm*    m_pAcroForms;
 
  private:
     // Prevent use of copy constructor and assignment operator.  These methods
@@ -587,15 +685,10 @@ class PODOFO_DOC_API PdfDocument {
     PdfDocument& operator=(const PdfDocument&);
 
     PdfVecObjects   m_vecObjects;
+   
 
-    PdfObject*      m_pTrailer;
-    PdfObject*      m_pCatalog;
-
-    PdfInfo*        m_pInfo;
     PdfOutlines*    m_pOutlines;
     PdfNamesTree*   m_pNamesTree;
-    PdfPagesTree*   m_pPagesTree;
-    PdfAcroForm*    m_pAcroForms;
 
     EPdfVersion     m_eVersion;
 };
@@ -675,6 +768,14 @@ inline FT_Library PdfDocument::GetFontLibrary() const
     return this->m_fontCache.GetFontLibrary();
 }
 
+// -----------------------------------------------------
+// 
+// -----------------------------------------------------
+inline void PdfDocument::SetFontConfigWrapper(const PdfFontConfigWrapper & rFontConfig)
+{
+    m_fontCache.SetFontConfigWrapper(rFontConfig);
+}
+
 };
 
 
