@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfFontCID.h"
@@ -31,6 +44,10 @@
 
 #include "PdfFontMetricsFreetype.h"
 
+#include "PdfFontTTFSubset.h"
+#include "base/PdfInputDevice.h"
+#include "base/PdfOutputDevice.h"
+
 #include <ft2build.h>
 #include FT_FREETYPE_H
 
@@ -43,23 +60,22 @@ struct TBFRange {
     std::vector<int> vecDest;
 };
 
+PdfFontCID::PdfFontCID( PdfFontMetrics* pMetrics, const PdfEncoding* const pEncoding, PdfObject* pObject, bool )
+    : PdfFont( pMetrics, pEncoding, pObject ), m_pDescendantFonts( NULL )
+{
+    /* this->Init( bEmbed, false ); */
+}
+
 PdfFontCID::PdfFontCID( PdfFontMetrics* pMetrics, const PdfEncoding* const pEncoding, 
-                        PdfVecObjects* pParent, bool bEmbed )
-    : PdfFont( pMetrics, pEncoding, pParent )
+                        PdfVecObjects* pParent, bool bEmbed, bool bSubset )
+    : PdfFont( pMetrics, pEncoding, pParent ), m_pDescendantFonts( NULL )
 {
-    this->Init( bEmbed );
+    this->Init( bEmbed, bSubset );
 }
 
-PdfFontCID::PdfFontCID( PdfFontMetrics* pMetrics, const PdfEncoding* const pEncoding, PdfObject* pObject, bool bEmbed )
-    : PdfFont( pMetrics, pEncoding, pObject )
-{
-    this->Init( bEmbed );
-}
-
-void PdfFontCID::Init( bool bEmbed )
+void PdfFontCID::Init( bool bEmbed, bool bSubset )
 {
     PdfObject* pDescriptor;
-    PdfObject* pDescendantFonts;
     PdfObject* pCIDSystemInfo;
     PdfObject* pUnicode;
 
@@ -67,7 +83,7 @@ void PdfFontCID::Init( bool bEmbed )
     PdfArray   array;
 
     // The descendant font is a CIDFont:
-    pDescendantFonts = this->GetObject()->GetOwner()->CreateObject("Font");
+    m_pDescendantFonts = this->GetObject()->GetOwner()->CreateObject("Font");
     pCIDSystemInfo   = this->GetObject()->GetOwner()->CreateObject();
     pDescriptor      = this->GetObject()->GetOwner()->CreateObject("FontDescriptor");
     pUnicode         = this->GetObject()->GetOwner()->CreateObject(); // The ToUnicode CMap
@@ -81,25 +97,25 @@ void PdfFontCID::Init( bool bEmbed )
     m_pEncoding->AddToDictionary( this->GetObject()->GetDictionary() );
 
     // The DecendantFonts, should be an indirect object:
-    array.push_back( pDescendantFonts->Reference() );
+    array.push_back( m_pDescendantFonts->Reference() );
     this->GetObject()->GetDictionary().AddKey( "DescendantFonts", array );
 
     // Setting the DescendantFonts paras
     // This is a type2 CIDFont, which is also known as TrueType:
-    pDescendantFonts->GetDictionary().AddKey( PdfName::KeySubtype, PdfName("CIDFontType2") );
+    m_pDescendantFonts->GetDictionary().AddKey( PdfName::KeySubtype, PdfName("CIDFontType2") );
 
     // Same base font as the owner font:
-    pDescendantFonts->GetDictionary().AddKey( "BaseFont", this->GetBaseFont() );
+    m_pDescendantFonts->GetDictionary().AddKey( "BaseFont", this->GetBaseFont() );
 
     // The CIDSystemInfo, should be an indirect object:
-    pDescendantFonts->GetDictionary().AddKey( "CIDSystemInfo", pCIDSystemInfo->Reference() );
+    m_pDescendantFonts->GetDictionary().AddKey( "CIDSystemInfo", pCIDSystemInfo->Reference() );
 
     // The FontDescriptor, should be an indirect object:
-    pDescendantFonts->GetDictionary().AddKey( "FontDescriptor", pDescriptor->Reference() );
-    pDescendantFonts->GetDictionary().AddKey( "CIDToGIDMap", PdfName("Identity") );
+    m_pDescendantFonts->GetDictionary().AddKey( "FontDescriptor", pDescriptor->Reference() );
+    m_pDescendantFonts->GetDictionary().AddKey( "CIDToGIDMap", PdfName("Identity") );
 
     // Add the width keys
-    this->CreateWidth( pDescendantFonts );
+    this->CreateWidth( m_pDescendantFonts );
 
     // Create the ToUnicode CMap
     this->CreateCMap( pUnicode );
@@ -107,7 +123,7 @@ void PdfFontCID::Init( bool bEmbed )
     // Setting the CIDSystemInfo paras:
     pCIDSystemInfo->GetDictionary().AddKey( "Registry", PdfString("Adobe") );
     pCIDSystemInfo->GetDictionary().AddKey( "Ordering", PdfString("Identity") );
-    pCIDSystemInfo->GetDictionary().AddKey( "Supplement", PdfVariant(static_cast<pdf_int64>(0LL)) );
+    pCIDSystemInfo->GetDictionary().AddKey( "Supplement", PdfVariant(static_cast<pdf_int64>(PODOFO_LL_LITERAL(0))) );
 
 
     // Setting the FontDescriptor paras:
@@ -115,21 +131,25 @@ void PdfFontCID::Init( bool bEmbed )
     m_pMetrics->GetBoundingBox( array );
 
     pDescriptor->GetDictionary().AddKey( "FontName", this->GetBaseFont() );
-    pDescriptor->GetDictionary().AddKey( PdfName::KeyFlags, PdfVariant( static_cast<pdf_int64>(32LL) ) ); // TODO: 0 ????
+    pDescriptor->GetDictionary().AddKey( PdfName::KeyFlags, PdfVariant( static_cast<pdf_int64>(PODOFO_LL_LITERAL(32)) ) ); // TODO: 0 ????
     pDescriptor->GetDictionary().AddKey( "FontBBox", array );
     pDescriptor->GetDictionary().AddKey( "ItalicAngle", PdfVariant( static_cast<pdf_int64>(m_pMetrics->GetItalicAngle()) ) );
     pDescriptor->GetDictionary().AddKey( "Ascent", m_pMetrics->GetPdfAscent() );
     pDescriptor->GetDictionary().AddKey( "Descent", m_pMetrics->GetPdfDescent() );
     pDescriptor->GetDictionary().AddKey( "CapHeight", m_pMetrics->GetPdfAscent() ); // m_pMetrics->CapHeight() );
-    pDescriptor->GetDictionary().AddKey( "StemV", PdfVariant( static_cast<pdf_int64>(1LL) ) );               // m_pMetrics->StemV() );
+    pDescriptor->GetDictionary().AddKey( "StemV", PdfVariant( static_cast<pdf_int64>(PODOFO_LL_LITERAL(1)) ) );               // m_pMetrics->StemV() );
 
     // Peter Petrov 24 September 2008
     m_pDescriptor = pDescriptor;
     
-    if( bEmbed )
+	 m_bIsSubsetting = bSubset;
+    if( bEmbed && !bSubset)
     {
         this->EmbedFont( pDescriptor );
         m_bWasEmbedded = true;
+    } else if (!bEmbed) {
+        // it's not asked to be embedded, thus mark as embedded already, to not do that at PdfFontCID::EmbedFont()
+        m_bWasEmbedded = true;
     }
 }
 
@@ -142,13 +162,67 @@ void PdfFontCID::EmbedFont()
     }
 }
 
+void PdfFontCID::EmbedSubsetFont()
+{
+	EmbedFont();
+}
+
+void PdfFontCID::AddUsedSubsettingGlyphs (const PdfString &sText, long lStringLen)
+{
+	if (IsSubsetting()) {
+		PODOFO_ASSERT( sText.IsUnicode() );
+
+		const pdf_utf16be *uniChars = sText.GetUnicode();
+		for (long ii = 0; ii < lStringLen; ii++) {
+			m_setUsed.insert(uniChars[ii]);
+		}
+	}
+}
+
 void PdfFontCID::EmbedFont( PdfObject* pDescriptor )
 {
-    PdfObject* pContents;
-    pdf_long       lSize = 0;
+	bool fallback = true;
     
     m_bWasEmbedded = true;    
         
+	if (IsSubsetting() && !m_setUsed.empty()) {
+		PdfFontMetrics *metrics = GetFontMetrics2();
+
+		if (metrics && metrics->GetFontDataLen() && metrics->GetFontData()) {
+			PdfInputDevice input(metrics->GetFontData(), metrics->GetFontDataLen());
+			PdfRefCountedBuffer buffer;
+			PdfOutputDevice output(&buffer);
+        
+			PdfFontTTFSubset subset(&input, metrics, PdfFontTTFSubset::eFontFileType_TTF);
+
+			std::set<pdf_utf16be>::const_iterator it, end = m_setUsed.end();
+			for (it = m_setUsed.begin(); it != end; it++) {
+				subset.AddCharacter(*it);
+			}
+
+			subset.BuildFont( &output );
+
+			PdfObject *pContents;
+
+			pContents = this->GetObject()->GetOwner()->CreateObject();
+			if( !pContents ) {
+				PODOFO_RAISE_ERROR( ePdfError_InvalidHandle );
+			}
+
+			pDescriptor->GetDictionary().AddKey( "FontFile2", pContents->Reference() );
+
+			pdf_long lSize = buffer.GetSize();
+			pContents->GetDictionary().AddKey("Length1", PdfVariant(static_cast<pdf_int64>(lSize)));
+			pContents->GetStream()->Set(buffer.GetBuffer(), lSize);
+
+			fallback = false;
+		}
+	}
+
+	if (fallback) {
+		PdfObject* pContents;
+		pdf_long       lSize = 0;
+    
     pContents = this->GetObject()->GetOwner()->CreateObject();
     if( !pContents )
     {
@@ -183,6 +257,7 @@ void PdfFontCID::EmbedFont( PdfObject* pDescriptor )
         pContents->GetStream()->Set( &stream );
     }
 }
+}
 
 void PdfFontCID::CreateWidth( PdfObject* pFontDict ) const
 {
@@ -224,71 +299,75 @@ void PdfFontCID::CreateWidth( PdfObject* pFontDict ) const
         }
     }
 
-    // Now compact the array
-    std::ostringstream oss;
-    PdfArray array;
-    array.reserve( nMax - nMin + 1 );
+	if (nMax >= nMin) {
+        // Now compact the array
+        std::ostringstream oss;
+        PdfArray array;
+        array.reserve( nMax - nMin + 1 );
 
-    i = nMin;
-    double    dCurWidth  = pdWidth[i];
-    pdf_int64 lCurIndex  = i++;
-    pdf_int64 lCurLength = 1L;
-
-    for( ;i<=nMax;i++ )
-    {
-        if( static_cast<int>(pdWidth[i] - dCurWidth) == 0 )
-            ++lCurLength;
-        else
+        i = nMin;
+        double    dCurWidth  = pdWidth[i];
+        pdf_int64 lCurIndex  = i++;
+        pdf_int64 lCurLength = 1L;
+        
+        for( ;i<=nMax;i++ )
         {
-            if( lCurLength > 1 ) 
-            {
-                array.push_back( lCurIndex );
-				pdf_int64 temp = lCurIndex + lCurLength - 1;
-                array.push_back( temp ); 
-                array.push_back( dCurWidth ); 
-            }
+            if( static_cast<int>(pdWidth[i] - dCurWidth) == 0 )
+                ++lCurLength;
             else
             {
-                if( array.size() && array.back().IsArray() ) 
+                if( lCurLength > 1 ) 
                 {
-                    array.back().GetArray().push_back( dCurWidth );
+                    array.push_back( lCurIndex );
+                    pdf_int64 temp = lCurIndex + lCurLength - 1;
+                    array.push_back( temp ); 
+                    array.push_back( dCurWidth ); 
                 }
                 else
                 {
-                    PdfArray tmp;
-                    tmp.push_back( dCurWidth );
+                    if( array.size() && array.back().IsArray() ) 
+                    {
+                        array.back().GetArray().push_back( dCurWidth );
+                    }
+                    else
+                    {
+                        PdfArray tmp;
+                        tmp.push_back( dCurWidth );
                         
-                    array.push_back( lCurIndex );
-                    array.push_back( tmp );
+                        array.push_back( lCurIndex );
+                        array.push_back( tmp );
+                    }
                 }
+                
+                lCurIndex  = i;
+                lCurLength = 1L;
+                dCurWidth  = pdWidth[i];
             }
-
-            lCurIndex  = i;
-            lCurLength = 1L;
-            dCurWidth  = pdWidth[i];
         }
-    }
 
-    if (array.size() == 0) 
-    {
-	    array.push_back( lCurIndex = nMin );
-	    array.push_back( lCurIndex = nMax );
-	    array.push_back( dCurWidth ); 
+        if (array.size() == 0) 
+        {
+            array.push_back( lCurIndex = nMin );
+            array.push_back( lCurIndex = nMax );
+            array.push_back( dCurWidth ); 
+        }
+        
+        pFontDict->GetDictionary().AddKey( PdfName("W"), array ); 
     }
 
-    pFontDict->GetDictionary().AddKey( PdfName("W"), array ); 
-
     free( pdWidth );
 }
 
 void PdfFontCID::CreateCMap( PdfObject* pUnicode ) const
 {
+    PdfFontMetricsFreetype* pFreetype = dynamic_cast<PdfFontMetricsFreetype*>(m_pMetrics);
+	if (!pFreetype) return;
+
     int  nFirstChar = m_pEncoding->GetFirstChar();
     int  nLastChar  = m_pEncoding->GetLastChar();
 
     std::ostringstream oss;
 
-    PdfFontMetricsFreetype* pFreetype = dynamic_cast<PdfFontMetricsFreetype*>(m_pMetrics);
     FT_Face   face = pFreetype->GetFace();
     FT_ULong  charcode;                                              
     FT_UInt   gindex;                                                
@@ -483,6 +562,57 @@ void PdfFontCID::CreateCMap( PdfObject* pUnicode ) const
     pStream->Append( out.str().c_str(), out.str().length() );
 */
 
+void PdfFontCID::MaybeUpdateBaseFontKey(void)
+{
+   if (!m_pDescendantFonts) {
+      return;
+   }
+
+   const PdfFontMetricsFreetype *pFreetype = dynamic_cast<const PdfFontMetricsFreetype *>(this->GetFontMetrics());
+   if (!pFreetype) {
+      return;
+   }
+
+   std::string name = this->GetBaseFont().GetName();
+   if (this->IsBold() && this->IsItalic()) {
+      if (pFreetype->IsBold() && pFreetype->IsItalic()) {
+         return;
+      }
+      if (pFreetype->IsBold() && !pFreetype->IsItalic()) {
+         name += ",Italic";
+      } else if (!pFreetype->IsBold() && pFreetype->IsItalic()) {
+         name += ",Bold";
+      } else {
+         name += ",BoldItalic";
+      }
+   } else if (this->IsBold()) {
+      if (pFreetype->IsBold()) {
+         return;
+      }
+      name += ",Bold";
+   } else if (this->IsItalic()) {
+      if (pFreetype->IsItalic()) {
+         return;
+      }
+      name += ",Italic";
+   } else {
+      return;
+   }
+
+   m_pDescendantFonts->GetDictionary().AddKey("BaseFont", PdfName( name ) );
+}
+
+void PdfFontCID::SetBold( bool bBold )
+{
+   PdfFont::SetBold(bBold);
+   MaybeUpdateBaseFontKey();
+}
+
+void PdfFontCID::SetItalic( bool bItalic )
+{
+   PdfFont::SetItalic(bItalic);
+   MaybeUpdateBaseFontKey();
+}
 
 };
 
