@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #ifndef _PDF_FONT_METRICS_H_
@@ -62,11 +75,11 @@ class PODOFO_DOC_API PdfFontMetrics {
      *  \param pszGlyphname name of the glyph
      *  \returns the width of a single named glyph
      */
-	virtual double GetGlyphWidth( const char* pszGlyphname ) const = 0;
+    virtual double GetGlyphWidth( const char* pszGlyphname ) const = 0;
 
     /** Create the bounding box array as required by the PDF reference
      *  so that it can be written directly to a PDF file.
-     * 
+     *
      *  \param array write the bounding box to this array.
      */
     virtual void GetBoundingBox( PdfArray & array ) const = 0;
@@ -137,7 +150,7 @@ class PODOFO_DOC_API PdfFontMetrics {
     inline unsigned long StringWidthMM( const wchar_t* pszText, unsigned int nLength = 0 ) const;
 #endif
 #endif
-    
+
     /** Retrieve the width of the given character in PDF units in the current font
      *  \param c character
      *  \returns the width in PDF units
@@ -167,13 +180,13 @@ class PODOFO_DOC_API PdfFontMetrics {
      */
     inline unsigned long GetLineSpacingMM() const;
 
-    /** Get the width of the underline for the current 
+    /** Get the width of the underline for the current
      *  font size in PDF units
      *  \returns the thickness of the underline in PDF units
      */
     virtual double GetUnderlineThickness() const = 0;
 
-    /** Get the width of the underline for the current 
+    /** Get the width of the underline for the current
      *  font size in 1/1000th mm
      *  \returns the thickness of the underline in 1/1000th mm
      */
@@ -203,13 +216,13 @@ class PODOFO_DOC_API PdfFontMetrics {
      */
     inline unsigned long GetStrikeOutPositionMM() const;
 
-    /** Get the width of the strikeout for the current 
+    /** Get the width of the strikeout for the current
      *  font size in PDF units
      *  \returns the thickness of the strikeout in PDF units
      */
     virtual double GetStrikeoutThickness() const = 0;
 
-    /** Get the width of the strikeout for the current 
+    /** Get the width of the strikeout for the current
      *  font size in 1/1000th mm
      *  \returns the thickness of the strikeout in 1/1000th mm
      */
@@ -251,7 +264,7 @@ class PODOFO_DOC_API PdfFontMetrics {
      *  units for the current font size.
      *
      *  \returns the ascender for this font
-     *  
+     *
      *  \see GetPdfAscent
      */
     virtual double GetAscent() const = 0;
@@ -259,12 +272,12 @@ class PODOFO_DOC_API PdfFontMetrics {
     /** Get the ascent of this font
      *  Used to build the font dictionay
      *  \returns the ascender for this font
-     *  
+     *
      *  \see GetAscent
      */
     virtual double GetPdfAscent() const = 0;
 
-    /** Get the descent of this font in PDF 
+    /** Get the descent of this font in PDF
      *  units for the current font size.
      *  This value is usually negative!
      *
@@ -296,7 +309,7 @@ class PODOFO_DOC_API PdfFontMetrics {
      */
     inline void SetFontSize( float fSize );
 
-    /** Retrieve the current font size of this metrics object 
+    /** Retrieve the current font size of this metrics object
      *  \returns the current font size
      */
     inline float GetFontSize() const;
@@ -323,11 +336,21 @@ class PODOFO_DOC_API PdfFontMetrics {
      */
     inline float GetFontCharSpace() const;
 
-    /** 
+    /** Set the word spacing of this metrics object
+     *  \param fWordSpace word spacing in PDF units
+     */
+    inline void SetWordSpace( float fWordSpace );
+
+    /** Retrieve the current word spacing of this metrics object
+     *  \returns the current font word spacing in PDF units
+     */
+    inline float GetWordSpace() const;
+
+    /**
      *  \returns the fonttype of the loaded font
      */
     inline EPdfFontType GetFontType() const;
- 
+
     /** Get the glyph id for a unicode character
      *  in the current font.
      *
@@ -338,13 +361,13 @@ class PODOFO_DOC_API PdfFontMetrics {
 
     /** Symbol fonts do need special treatment in a few cases.
      *  Use this method to check if the current font is a symbol
-     *  font. Symbold fonts are detected by checking 
+     *  font. Symbold fonts are detected by checking
      *  if they use FT_ENCODING_MS_SYMBOL as internal encoding.
-     * 
+     *
      * \returns true if this is a symbol font
      */
     virtual bool IsSymbol() const = 0;
-    
+
     /** Try to detect the internal fonttype from
      *  the file extension of a fontfile.
      *
@@ -355,7 +378,7 @@ class PODOFO_DOC_API PdfFontMetrics {
     static EPdfFontType FontTypeFromFilename( const char* pszFilename );
 
  protected:
-    /** 
+    /**
      *  Set the fonttype.
      *  \param eFontType fonttype
      */
@@ -366,6 +389,7 @@ class PODOFO_DOC_API PdfFontMetrics {
     float         m_fFontSize;
     float         m_fFontScale;
     float         m_fFontCharSpace;
+    float         m_fWordSpace;
 
     std::vector<double> m_vecWidth;
 
@@ -374,7 +398,7 @@ class PODOFO_DOC_API PdfFontMetrics {
 };
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 unsigned long PdfFontMetrics::CharWidthMM( unsigned char c ) const
 {
@@ -382,7 +406,7 @@ unsigned long PdfFontMetrics::CharWidthMM( unsigned char c ) const
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 double PdfFontMetrics::StringWidth( const PdfString & rsString ) const
 {
@@ -390,7 +414,7 @@ double PdfFontMetrics::StringWidth( const PdfString & rsString ) const
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 unsigned long PdfFontMetrics::StringWidthMM( const char* pszText, unsigned int nLength ) const
 {
@@ -398,7 +422,7 @@ unsigned long PdfFontMetrics::StringWidthMM( const char* pszText, unsigned int n
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 unsigned long PdfFontMetrics::StringWidthMM( const pdf_utf16be* pszText, unsigned int nLength ) const
 {
@@ -406,7 +430,7 @@ unsigned long PdfFontMetrics::StringWidthMM( const pdf_utf16be* pszText, unsigne
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 #ifndef _WCHAR_T_DEFINED
 #if defined(_MSC_VER)  &&  _MSC_VER <= 1200			// nicht fr Visualstudio 6
@@ -419,7 +443,7 @@ unsigned long PdfFontMetrics::StringWidthMM( const wchar_t* pszText, unsigned in
 #endif
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 unsigned long PdfFontMetrics::GetLineSpacingMM() const
 {
@@ -427,7 +451,7 @@ unsigned long PdfFontMetrics::GetLineSpacingMM() const
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 long PdfFontMetrics::GetUnderlinePositionMM() const
 {
@@ -435,15 +459,15 @@ long PdfFontMetrics::GetUnderlinePositionMM() const
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 unsigned long PdfFontMetrics::GetStrikeOutPositionMM() const
 {
-	return static_cast<long>(this->GetStrikeOutPosition() /  PODOFO_CONVERSION_CONSTANT);
+    return static_cast<long>(this->GetStrikeOutPosition() /  PODOFO_CONVERSION_CONSTANT);
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 unsigned long PdfFontMetrics::GetUnderlineThicknessMM() const
 {
@@ -451,7 +475,7 @@ unsigned long PdfFontMetrics::GetUnderlineThicknessMM() const
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 unsigned long PdfFontMetrics::GetStrikeoutThicknessMM() const
 {
@@ -459,7 +483,7 @@ unsigned long PdfFontMetrics::GetStrikeoutThicknessMM() const
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 const char* PdfFontMetrics::GetFilename() const
 {
@@ -467,15 +491,15 @@ const char* PdfFontMetrics::GetFilename() const
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 EPdfFontType PdfFontMetrics::GetFontType() const
 {
     return m_eFontType;
 }
-  
+
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 void PdfFontMetrics::SetFontType(EPdfFontType eFontType)
 {
@@ -483,7 +507,7 @@ void PdfFontMetrics::SetFontType(EPdfFontType eFontType)
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 float PdfFontMetrics::GetFontSize() const
 {
@@ -491,7 +515,7 @@ float PdfFontMetrics::GetFontSize() const
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 void PdfFontMetrics::SetFontSize( float fSize )
 {
@@ -499,15 +523,15 @@ void PdfFontMetrics::SetFontSize( float fSize )
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 float PdfFontMetrics::GetFontScale() const
 {
     return m_fFontScale;
 }
- 
+
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 float PdfFontMetrics::GetFontCharSpace() const
 {
@@ -515,7 +539,15 @@ float PdfFontMetrics::GetFontCharSpace() const
 }
 
 // -----------------------------------------------------
-// 
+//
+// -----------------------------------------------------
+inline float PdfFontMetrics::GetWordSpace() const
+{
+    return m_fWordSpace;
+}
+
+// -----------------------------------------------------
+//
 // -----------------------------------------------------
 const char* PdfFontMetrics::GetSubsetFontnamePrefix() const
 {
@@ -523,7 +555,7 @@ const char* PdfFontMetrics::GetSubsetFontnamePrefix() const
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 void PdfFontMetrics::SetFontScale( float fScale )
 {
@@ -531,13 +563,21 @@ void PdfFontMetrics::SetFontScale( float fScale )
 }
 
 // -----------------------------------------------------
-// 
+//
 // -----------------------------------------------------
 void PdfFontMetrics::SetFontCharSpace( float fCharSpace )
 {
     m_fFontCharSpace = fCharSpace;
 }
 
+// -----------------------------------------------------
+//
+// -----------------------------------------------------
+inline void PdfFontMetrics::SetWordSpace( float fWordSpace )
+{
+    m_fWordSpace = fWordSpace;
+}
+
 
 };
 
