@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfFontTTFSubset.h"
@@ -153,7 +166,7 @@ void PdfFontTTFSubset::GetNumberOfGlyphs()
     GetData( ulMaxpOffset+__LENGTH_DWORD*1,&m_numGlyphs,__LENGTH_DWORD);
     m_numGlyphs = Big2Little(m_numGlyphs);
 
-    std::cout << "Number of Glyphs:	"<< m_numGlyphs << std::endl;
+    //std::cout << "Number of Glyphs:	"<< m_numGlyphs << std::endl;
 }
 
 void PdfFontTTFSubset::InitTables()
@@ -181,8 +194,13 @@ void PdfFontTTFSubset::InitTables()
         tbl.m_length = Big2Little(tbl.m_length);
 
         //It seems that the EBDT, EBLC, EBSC table can be erased.
-        if (tbl.m_strTableName == "EBDT")
-            /*||tbl.m_strTableName == string("EBLC")||tbl.m_strTableName == string("EBSC")*/
+        if (PoDoFo::compat::strcasecmp(tbl.m_strTableName.c_str(), "EBDT") == 0
+            /*|| PoDoFo::compat::strcasecmp(tbl.m_strTableName.c_str(), "EBLC") == 0
+            || PoDoFo::compat::strcasecmp(tbl.m_strTableName.c_str(), "EBSC") == 0*/
+            || PoDoFo::compat::strcasecmp(tbl.m_strTableName.c_str(), "DSIG") == 0
+            || PoDoFo::compat::strcasecmp(tbl.m_strTableName.c_str(), "LTSH") == 0
+            || PoDoFo::compat::strcasecmp(tbl.m_strTableName.c_str(), "VDMX") == 0
+            || PoDoFo::compat::strcasecmp(tbl.m_strTableName.c_str(), "HDMX") == 0)
         {
             continue;
         }
@@ -282,12 +300,12 @@ void PdfFontTTFSubset::BuildFont( PdfOutputDevice* pOutputDevice )
     std::vector<TTrueTypeTable> vOldTable = m_vTable;	//vOldTable will be used later.
 
     //Print the old table information:
-    for ( i = 0; i < m_numTables; i++)
-    {
-        std::cout << "OldTable:\t" << m_vTable[i].m_strTableName << std::endl;
-        std::cout << "\tOffSet:\t" << m_vTable[i].m_offset << std::endl;
-        std::cout << "\t\tLength:\t"<<m_vTable[i].m_length << std::endl;
-    }
+//    for ( i = 0; i < m_numTables; i++)
+//    {
+//        std::cout << "OldTable:\t" << m_vTable[i].m_strTableName << std::endl;
+//        std::cout << "\tOffSet:\t" << m_vTable[i].m_offset << std::endl;
+//        std::cout << "\t\tLength:\t"<<m_vTable[i].m_length << std::endl;
+//    }
 	
     //Change the offsets:
     m_vTable[0].m_offset = __LENGTH_HEADER12+__LENGTH_OFFSETTABLE16*m_numTables;
@@ -299,12 +317,12 @@ void PdfFontTTFSubset::BuildFont( PdfOutputDevice* pOutputDevice )
     }
 	
     //Print the new table information:
-    for ( i = 0; i < m_numTables; i++)
-    {
-        std::cout << "NewTable:\t" << m_vTable[i].m_strTableName <<std::endl;
-        std::cout << "\tOffSet:\t" << m_vTable[i].m_offset <<std::endl;
-        std::cout << "\t\tLength:\t"<<m_vTable[i].m_length <<std::endl;
-    }
+//    for ( i = 0; i < m_numTables; i++)
+//    {
+//        std::cout << "NewTable:\t" << m_vTable[i].m_strTableName <<std::endl;
+//        std::cout << "\tOffSet:\t" << m_vTable[i].m_offset <<std::endl;
+//        std::cout << "\t\tLength:\t"<<m_vTable[i].m_length <<std::endl;
+//    }
 
     //The glyph data:
     if (m_bIsLongLoca)
