@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfFontType1.h"
@@ -34,13 +47,14 @@
 namespace PoDoFo {
 
 PdfFontType1::PdfFontType1( PdfFontMetrics* pMetrics, const PdfEncoding* const pEncoding, 
-                            PdfVecObjects* pParent, bool bEmbed )
+                            PdfVecObjects* pParent, bool bEmbed, bool bSubsetting )
     : PdfFontSimple( pMetrics, pEncoding, pParent )
 {
 	memset( m_bUsed, 0, sizeof( m_bUsed ) );
+	m_bIsSubsetting = bSubsetting;
     this->Init( bEmbed, PdfName("Type1") );
 }
- 
+
 
 PdfFontType1::PdfFontType1( PdfFontMetrics* pMetrics, const PdfEncoding* const pEncoding, 
                             PdfObject* pObject )
@@ -56,6 +70,10 @@ PdfFontType1::PdfFontType1( PdfFontType1* pFont, PdfFontMetrics* pMetrics, const
 	// don't embedd font
     Init( false, PdfName("Type1") );
 
+	// Use identical subset-names
+	if ( pFont->IsSubsetting() )
+		GetObject()->GetDictionary().AddKey( "BaseFont", pFont->GetObject()->GetDictionary().GetKey( "BaseFont" ) );
+
 	// set identifier
 	std::string id = pFont->GetIdentifier().GetName();
 	id += pszSuffix;
@@ -102,6 +120,7 @@ void PdfFontType1::EmbedSubsetFont()
     PdfObject*  pContents;
     const char* pBuffer;
     char*       pAllocated = NULL;
+    int         i;
 
     m_bWasEmbedded = true;
 
@@ -167,11 +186,8 @@ void PdfFontType1::EmbedSubsetFont()
 	// transfer ascii-part, modify encoding dictionary (dup ...), if present
 	std::string line;
 	bool dupFound = false;
-	for ( int i = 0; i < length; i++ )
+	for ( i = 0; i < length; i++ )
 	{
-		// TODO: only embed with used glyphs
-		EmbedFontFile( m_pDescriptor );
-		m_bWasEmbedded = true;
 		line += static_cast<char>( inBuff[inIndex+i] );
 		if ( inBuff[inIndex+i] == '\r' )
 		{
@@ -199,7 +215,11 @@ void PdfFontType1::EmbedSubsetFont()
 					dupFound = true;
 				}
 			}
+#if defined(_MSC_VER)  &&  _MSC_VER <= 1200 // Visual Studio 6
+			line.erase();
+#else
 			line.clear();
+#endif
 		}
 	}
 	inIndex += length;
@@ -225,7 +245,11 @@ void PdfFontType1::EmbedSubsetFont()
 		PdfType1EncryptEexec inCrypt;
 		
 		outIndex = outIndexStart;
+#if defined(_MSC_VER)  &&  _MSC_VER <= 1200 // Visual Studio 6
+		line.erase();
+#else
 		line.clear();
+#endif
 		foundSeacGlyph = false;
 		bool inCharString = false;
 		for ( int i = 0; i < length;  )
@@ -325,14 +349,18 @@ void PdfFontType1::EmbedSubsetFont()
 				// parse for /CharStrings = begin of glyphs
 				if ( line.find( "/CharStrings" ) != static_cast<size_t>(-1) )
 					inCharString = true;
+#if defined(_MSC_VER)  &&  _MSC_VER <= 1200 // Visual Studio 6
+				line.erase();
+#else
 				line.clear();
+#endif
 			}
 		}
 	} while ( foundSeacGlyph );
 
 	// now encrypt resulting output-buffer
 	PdfType1EncryptEexec outCrypt;
-	for ( int i = outIndexStart; i < outIndex; i++ )
+	for ( i = outIndexStart; i < outIndex; i++ )
 		outBuff[i] = outCrypt.Encrypt( outBuff[i] );
 
 	lLength2 = outIndex - outIndexStart;
