@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #include "PdfPage.h" 
@@ -26,6 +39,7 @@
 #include "base/PdfRect.h"
 #include "base/PdfVariant.h"
 #include "base/PdfWriter.h"
+#include "base/PdfStream.h"
 
 #include "PdfDocument.h"
 
@@ -270,6 +284,14 @@ int PdfPage::GetRotation() const
     return rot;
 }
 
+void PdfPage::SetRotation(int nRotation)
+{
+    if( nRotation != 0 && nRotation != 90 && nRotation != 180 && nRotation != 270 )
+        PODOFO_RAISE_ERROR( ePdfError_ValueOutOfRange );
+
+    this->GetObject()->GetDictionary().AddKey( "Rotate", PdfVariant(static_cast<pdf_int64>(nRotation)) );
+}
+
 PdfObject* PdfPage::GetAnnotationsArray( bool bCreate ) const
 {
     PdfObject* pObj;
@@ -491,22 +513,31 @@ unsigned int PdfPage::GetPageNumber() const
 
     while( pParent ) 
     {
-        const PdfArray& kids        = pParent->GetIndirectKey( "Kids" )->GetArray();
-        PdfArray::const_iterator it = kids.begin();
-
-        while( it != kids.end() && (*it).GetReference() != ref )
+        PdfObject* pKids = pParent->GetIndirectKey( "Kids" );
+        if ( pKids != NULL )
         {
-            PdfObject* pNode = this->GetObject()->GetOwner()->GetObject( (*it).GetReference() );
+            const PdfArray& kids        = pKids->GetArray();
+            PdfArray::const_iterator it = kids.begin();
 
-            if( pNode->GetDictionary().GetKey( PdfName::KeyType )->GetName() == PdfName( "Pages" ) )
-                nPageNumber += static_cast<int>(pNode->GetDictionary().GetKey( "Count" )->GetNumber());
-            else 
-                // if we do not have a page tree node, 
-                // we most likely have a page object:
-                // so the page count is 1
-                ++nPageNumber;
+            while( it != kids.end() && (*it).GetReference() != ref )
+            {
+                PdfObject* pNode = this->GetObject()->GetOwner()->GetObject( (*it).GetReference() );
 
-            ++it;
+                if( pNode->GetDictionary().GetKey( PdfName::KeyType ) != NULL 
+                    && pNode->GetDictionary().GetKey( PdfName::KeyType )->GetName() == PdfName( "Pages" ) )
+                {
+                    PdfObject* pCount = pNode->GetIndirectKey( "Count" );
+                    if( pCount != NULL ) {
+                        nPageNumber += static_cast<int>(pCount->GetNumber());
+                    }
+                } else {
+                    // if we do not have a page tree node, 
+                    // we most likely have a page object:
+                    // so the page count is 1
+                    ++nPageNumber;
+                }
+                ++it;
+            }
         }
 
         ref     = pParent->Reference();
@@ -525,8 +556,7 @@ int PdfPage::GetNumFields() const
     {
         pAnnot = const_cast<PdfPage*>(this)->GetAnnotation( i );
         // Count every widget annotation with a FieldType as PdfField
-        if( pAnnot->GetType() == ePdfAnnotation_Widget && 
-            pAnnot->GetObject()->GetDictionary().HasKey( PdfName("FT") ) )
+        if( pAnnot->GetType() == ePdfAnnotation_Widget )
             ++nCount;
     }
 
@@ -542,8 +572,7 @@ PdfField PdfPage::GetField( int index )
     {
         pAnnot = this->GetAnnotation( i );
         // Count every widget annotation with a FieldType as PdfField
-        if( pAnnot->GetType() == ePdfAnnotation_Widget && 
-            pAnnot->GetObject()->GetDictionary().HasKey( PdfName("FT") ) )
+        if( pAnnot->GetType() == ePdfAnnotation_Widget )
         {
             if( nCount == index )
             {
@@ -580,4 +609,41 @@ PdfObject* PdfPage::GetFromResources( const PdfName & rType, const PdfName & rKe
     return NULL;
 }
 
+
+PdfObject* PdfPage::GetOwnAnnotationsArray( bool bCreate, PdfDocument *pDocument)
+{
+   PdfObject* pObj;
+
+   if ( this->GetObject()->GetDictionary().HasKey( "Annots" ) )  {
+      pObj = this->GetObject()->GetIndirectKey( "Annots" );
+        
+      if(!pObj) {
+         pObj = this->GetObject()->GetDictionary().GetKey("Annots");
+         if( pObj->IsReference() ) {
+            if( !pDocument ) {
+               PODOFO_RAISE_ERROR_INFO( ePdfError_InvalidHandle, "Object is a reference but does not have an owner!" );
+            }
+
+            pObj = pDocument->GetObjects()->GetObject( pObj->GetReference() );
+            if(pObj) {
+               pObj->SetOwner(this->GetObject()->GetOwner());
+            }
+         } else
+            pObj->SetOwner( this->GetObject()->GetOwner() );// even directs might want an owner...
+      }
+
+      if( pObj && pObj->IsArray() )
+         return pObj;
+    }
+    else if( bCreate ) 
+    {
+        PdfArray array;
+        this->GetNonConstObject()->GetDictionary().AddKey( "Annots", array );
+        return const_cast<PdfObject*>(this->GetObject()->GetDictionary().GetKey( "Annots" ));
+    }
+
+    return NULL;
+}
+
+
 };
