@@ -1,22 +1,35 @@
 /***************************************************************************
-*   Copyriht (C) 2006 by Dominik Seichter                                *
-*   domseichter@web.de                                                    *
-*                                                                         *
-*   This program is free software; you can redistribute it and/or modify  *
-*   it under the terms of the GNU Library General Public License as       *
-*   published by the Free Software Foundation; either version 2 of the    *
-*   License, or (at your option) any later version.                       *
-*                                                                         *
-*   This program is distributed in the hope that it will be useful,       *
-*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
-*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
-*   GNU General Public License for more details.                          *
-*                                                                         *
-*   You should have received a copy of the GNU Library General Public     *
-*   License along with this program; if not, write to the                 *
-*   Free Software Foundation, Inc.,                                       *
-*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
-***************************************************************************/
+ *   Copyriht (C) 2006 by Dominik Seichter                                *
+ *   domseichter@web.de                                                    *
+ *                                                                         *
+ *   This program is free software; you can redistribute it and/or modify  *
+ *   it under the terms of the GNU Library General Public License as       *
+ *   published by the Free Software Foundation; either version 2 of the    *
+ *   License, or (at your option) any later version.                       *
+ *                                                                         *
+ *   This program is distributed in the hope that it will be useful,       *
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
+ *   GNU General Public License for more details.                          *
+ *                                                                         *
+ *   You should have received a copy of the GNU Library General Public     *
+ *   License along with this program; if not, write to the                 *
+ *   Free Software Foundation, Inc.,                                       *
+ *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
+ ***************************************************************************/
 
 #include "PdfPagesTree.h"
 
@@ -38,12 +51,12 @@ PdfPagesTree::PdfPagesTree( PdfVecObjects* pParent )
       m_cache( 0 )
 {
     GetObject()->GetDictionary().AddKey( "Kids", PdfArray() ); // kids->Reference() 
-    GetObject()->GetDictionary().AddKey( "Count", PdfObject( static_cast<pdf_int64>(0LL) ) );
+    GetObject()->GetDictionary().AddKey( "Count", PdfObject( static_cast<pdf_int64>(PODOFO_LL_LITERAL(0)) ) );
 }
 
 PdfPagesTree::PdfPagesTree( PdfObject* pPagesRoot )
     : PdfElement( "Pages", pPagesRoot ),
-      m_cache( static_cast<int>(pPagesRoot->GetDictionary().GetKeyAsLong( "Count", static_cast<pdf_int64>(0LL) )) )
+      m_cache( GetChildCount( pPagesRoot ) )
 {
     if( !this->GetObject() ) 
     {
@@ -58,8 +71,7 @@ PdfPagesTree::~PdfPagesTree()
 
 int PdfPagesTree::GetTotalNumberOfPages() const
 {
-    return ( ( this->GetObject()->GetDictionary().HasKey( "Count" ) ) ?
-             static_cast<int>(this->GetObject()->GetDictionary().GetKeyAsLong( "Count", 0LL )) : 0 );
+    return GetChildCount( GetObject() );
 }
 
 PdfPage* PdfPagesTree::GetPage( int nIndex )
@@ -103,33 +115,33 @@ PdfPage* PdfPagesTree::GetPage( const PdfReference & ref )
 }
 
 
-void PdfPagesTree::InsertPage( int inAfterPageNumber, PdfPage* inPage )
+void PdfPagesTree::InsertPage( int nAfterPageIndex, PdfPage* inPage )
 {
-    this->InsertPage( inAfterPageNumber, inPage->GetObject() );
+    this->InsertPage( nAfterPageIndex, inPage->GetObject() );
 }
 
-void PdfPagesTree::InsertPage( int nAfterPageNumber, PdfObject* pPage )
+void PdfPagesTree::InsertPage( int nAfterPageIndex, PdfObject* pPage )
 {
     bool bInsertBefore = false;
 
-    if( ePdfPageInsertionPoint_InsertBeforeFirstPage == nAfterPageNumber )
+    if( ePdfPageInsertionPoint_InsertBeforeFirstPage == nAfterPageIndex )
     {
         bInsertBefore = true;
-        nAfterPageNumber = 0;
+        nAfterPageIndex = 0;
     }
-    else if( nAfterPageNumber < 0 ) 
+    else if( nAfterPageIndex < 0 ) 
     {
         // Only ePdfPageInsertionPoint_InsertBeforeFirstPage is valid here
         PdfError::LogMessage( eLogSeverity_Information,
                               "Invalid argument to PdfPagesTree::InsertPage: %i (Only ePdfPageInsertionPoint_InsertBeforeFirstPage is valid here).",
-                              nAfterPageNumber );
+                              nAfterPageIndex );
         return;
     }
 
-    //printf("Fetching page node: %i\n", nAfterPageNumber);
+    //printf("Fetching page node: %i\n", nAfterPageIndex);
     PdfObjectList lstParents;
-    //printf("Searching page=%i\n", nAfterPageNumber );
-    PdfObject* pPageBefore = this->GetPageNode( nAfterPageNumber, this->GetRoot(), lstParents );
+    //printf("Searching page=%i\n", nAfterPageIndex );
+    PdfObject* pPageBefore = this->GetPageNode( nAfterPageIndex, this->GetRoot(), lstParents );
     
     //printf("pPageBefore=%p lstParents=%i\n", pPageBefore,lstParents.size() );
     if( !pPageBefore || lstParents.size() == 0 ) 
@@ -138,7 +150,7 @@ void PdfPagesTree::InsertPage( int nAfterPageNumber, PdfObject* pPage )
         {
             PdfError::LogMessage( eLogSeverity_Critical,
                                   "Cannot find page %i or page %i has no parents. Cannot insert new page.",
-                                  nAfterPageNumber, nAfterPageNumber );
+                                  nAfterPageIndex, nAfterPageIndex );
             return;
         }
         else
@@ -160,20 +172,97 @@ void PdfPagesTree::InsertPage( int nAfterPageNumber, PdfObject* pPage )
         InsertPageIntoNode( pParent, lstParents, nKidsIndex, pPage );
     }
 
-    m_cache.InsertPage( nAfterPageNumber );
+    m_cache.InsertPage( (bInsertBefore && nAfterPageIndex == 0) ? ePdfPageInsertionPoint_InsertBeforeFirstPage : nAfterPageIndex );
 }
 
+void PdfPagesTree::InsertPages( int nAfterPageIndex, const std::vector<PdfObject*>& vecPages )
+{
+    bool bInsertBefore = false;
+    if( ePdfPageInsertionPoint_InsertBeforeFirstPage == nAfterPageIndex )
+    {
+        bInsertBefore = true;
+        nAfterPageIndex = 0;
+    }
+    else if( nAfterPageIndex < 0 ) 
+    {
+        // Only ePdfPageInsertionPoint_InsertBeforeFirstPage is valid here
+        PdfError::LogMessage( eLogSeverity_Information,
+                              "Invalid argument to PdfPagesTree::InsertPage: %i (Only ePdfPageInsertionPoint_InsertBeforeFirstPage is valid here).",
+                              nAfterPageIndex );
+        return;
+    }
+
+    PdfObjectList lstParents;
+    PdfObject* pPageBefore = this->GetPageNode( nAfterPageIndex, this->GetRoot(), lstParents );
+    
+    if( !pPageBefore || lstParents.size() == 0 ) 
+    {
+        if( this->GetTotalNumberOfPages() != 0 ) 
+        {
+            PdfError::LogMessage( eLogSeverity_Critical,
+                                  "Cannot find page %i or page %i has no parents. Cannot insert new page.",
+                                  nAfterPageIndex, nAfterPageIndex );
+            return;
+        }
+        else
+        {
+            // We insert the first page into an empty pages tree
+            PdfObjectList lstPagesTree;
+            lstPagesTree.push_back( this->GetObject() );
+            // Use -1 as index to insert before the empty kids array
+            InsertPagesIntoNode( this->GetObject(), lstPagesTree, -1, vecPages );
+        }
+    }
+    else
+    {
+        PdfObject* pParent = lstParents.back();
+        int nKidsIndex = bInsertBefore  ? -1 : this->GetPosInKids( pPageBefore, pParent );
+
+        InsertPagesIntoNode( pParent, lstParents, nKidsIndex, vecPages );
+    }
+
+    m_cache.InsertPages( (bInsertBefore && nAfterPageIndex == 0) ? ePdfPageInsertionPoint_InsertBeforeFirstPage : nAfterPageIndex,  vecPages.size() );
+}
 
 PdfPage* PdfPagesTree::CreatePage( const PdfRect & rSize )
 {
     PdfPage* pPage = new PdfPage( rSize, GetRoot()->GetOwner() );
 
     InsertPage( this->GetTotalNumberOfPages() - 1, pPage );
-    m_cache.AddPageObject( this->GetTotalNumberOfPages() - 1, pPage );
+    m_cache.AddPageObject( this->GetTotalNumberOfPages(), pPage );
     
     return pPage;
 }
 
+PdfPage* PdfPagesTree::InsertPage( const PdfRect & rSize, int atIndex)
+{
+    PdfPage* pPage = new PdfPage( rSize, GetRoot()->GetOwner() );
+
+	 if (atIndex < 0 || atIndex >= this->GetTotalNumberOfPages()) {
+		 atIndex = this->GetTotalNumberOfPages() - 1;
+	 }
+
+	 InsertPage( atIndex - 1, pPage );
+    m_cache.AddPageObject( atIndex, pPage );
+
+    return pPage;
+}
+
+void PdfPagesTree::CreatePages( const std::vector<PdfRect>& vecSizes )
+{
+    std::vector<PdfPage*> vecPages;
+    std::vector<PdfObject*> vecObjects;
+    for (std::vector<PdfRect>::const_iterator it = vecSizes.begin(); it != vecSizes.end(); ++it)
+    {
+        PdfPage* pPage = new PdfPage( (*it), GetRoot()->GetOwner() );
+        vecPages.push_back( pPage );
+        vecObjects.push_back( pPage->GetObject() );
+    }
+
+    InsertPages( this->GetTotalNumberOfPages() - 1, vecObjects );
+    m_cache.AddPageObjects( this->GetTotalNumberOfPages(), vecPages );
+}
+
 void PdfPagesTree::DeletePage( int nPageNumber )
 {
     // Delete from cache
@@ -227,7 +316,7 @@ PdfObject* PdfPagesTree::GetPageNode( int nPageNum, PdfObject* pParent,
 
     
     const PdfObject* pObj = pParent->GetIndirectKey( "Kids" );
-    if( !pObj->IsArray() )
+    if( pObj == NULL || !pObj->IsArray() )
     {
         PODOFO_RAISE_ERROR( ePdfError_InvalidDataType );
     }
@@ -236,7 +325,7 @@ PdfObject* PdfPagesTree::GetPageNode( int nPageNum, PdfObject* pParent,
     PdfArray::const_iterator it = rKidsArray.begin();
 
     const size_t numDirectKids = rKidsArray.size();
-    const size_t numKids = static_cast<size_t>(pParent->GetDictionary().GetKeyAsLong( "Count", 0LL ));
+    const size_t numKids = GetChildCount(pParent);
 
     if( static_cast<int>(numKids) < nPageNum ) 
     {
@@ -284,7 +373,7 @@ PdfObject* PdfPagesTree::GetPageNode( int nPageNum, PdfObject* pParent,
 
                 if( this->IsTypePages(pChild) ) 
                 {
-                    int childCount = this->GetChildCount( pChild );
+                    int childCount = GetChildCount( pChild );
                     if( childCount < nPageNum + 1 ) // Pages are 0 based, but count is not
                     {
                         // skip this page node
@@ -349,6 +438,9 @@ PdfObject* PdfPagesTree::GetPageNodeFromArray( int nPageNum, const PdfArray & rK
         }
 
         PdfObject* pgObject = GetRoot()->GetOwner()->GetObject( rVar.GetReference() );
+		if(pgObject==NULL) {
+			PODOFO_RAISE_ERROR_INFO( ePdfError_PageNotFound, "Invalid reference." );
+		}
         //printf("Reading %s\n", pgObject->Reference().ToString().c_str());
         // make sure the object is a /Page and not a /Pages with a single kid
         if( this->IsTypePage(pgObject) ) 
@@ -364,7 +456,10 @@ PdfObject* PdfPagesTree::GetPageNodeFromArray( int nPageNum, const PdfArray & rK
 
             rLstParents.push_back( pgObject );
             rVar = *(pgObject->GetDictionary().GetKey( "Kids" ));
-        }
+        } else {
+	  // Reference to unexpected object
+	  PODOFO_RAISE_ERROR_INFO( ePdfError_PageNotFound, "Reference to unexpected object." );
+	}
     }
 
     return NULL;
@@ -397,7 +492,13 @@ int PdfPagesTree::GetChildCount( const PdfObject* pNode ) const
     if( !pNode ) 
         return 0;
 
-    return static_cast<int>(pNode->GetDictionary().GetKeyAsLong("Count", 0L));
+    const PdfObject *pCount = pNode->GetIndirectKey( "Count" );
+    if( pCount != 0 ) {
+        return (pCount->GetDataType() == PoDoFo::ePdfDataType_Number) ?  
+            static_cast<int>( pCount->GetNumber() ):0;
+    } else {
+        return 0;
+    }
 }
 
 int PdfPagesTree::GetPosInKids( PdfObject* pPageObj, PdfObject* pPageParent )
@@ -487,6 +588,64 @@ void PdfPagesTree::InsertPageIntoNode( PdfObject* pParent, const PdfObjectList &
     pPage->GetDictionary().AddKey( PdfName("Parent"), pParent->Reference() );
 }
 
+void PdfPagesTree::InsertPagesIntoNode( PdfObject* pParent, const PdfObjectList & rlstParents, 
+                                       int nIndex, const std::vector<PdfObject*>& vecPages )
+{
+    if( !pParent || !vecPages.size() ) 
+    {
+        PODOFO_RAISE_ERROR( ePdfError_InvalidHandle );
+    }
+
+    // 1. Add the reference of the new page to the kids array of pParent
+    // 2. Increase count of every node in lstParents (which also includes pParent)
+    // 3. Add Parent key to the page
+
+    // 1. Add reference
+    const PdfArray oldKids = pParent->GetDictionary().GetKey( PdfName("Kids") )->GetArray();
+    PdfArray newKids;
+    newKids.reserve( oldKids.GetSize() + vecPages.size() );
+
+    bool bIsPushedIn = false;
+    int i=0;
+    for (PdfArray::const_iterator it=oldKids.begin(); it!=oldKids.end(); ++it, ++i ) 
+    {
+        if ( !bIsPushedIn && (nIndex < i) )    // Pushing before
+        {
+            for (std::vector<PdfObject*>::const_iterator itPages=vecPages.begin(); itPages!=vecPages.end(); ++itPages)
+            {
+                newKids.push_back( (*itPages)->Reference() );    // Push all new kids at once
+            }
+            bIsPushedIn = true;
+        }
+        newKids.push_back( *it );    // Push in the old kids
+    }
+
+    // If new kids are still not pushed in then they may be appending to the end
+    if ( !bIsPushedIn && ( (nIndex + 1) == static_cast<int>(oldKids.size())) ) 
+    {
+        for (std::vector<PdfObject*>::const_iterator itPages=vecPages.begin(); itPages!=vecPages.end(); ++itPages)
+        {
+            newKids.push_back( (*itPages)->Reference() );    // Push all new kids at once
+        }
+        bIsPushedIn = true;
+    }
+
+    pParent->GetDictionary().AddKey( PdfName("Kids"), newKids );
+ 
+
+    // 2. increase count
+    for ( PdfObjectList::const_reverse_iterator itParents = rlstParents.rbegin(); itParents != rlstParents.rend(); ++itParents )
+    {
+        this->ChangePagesCount( *itParents, vecPages.size() );
+    } 
+
+    // 3. add parent key to each of the pages
+    for (std::vector<PdfObject*>::const_iterator itPages=vecPages.begin(); itPages!=vecPages.end(); ++itPages)
+    {
+        (*itPages)->GetDictionary().AddKey( PdfName("Parent"), pParent->Reference() );
+    }
+}
+
 void PdfPagesTree::DeletePageFromNode( PdfObject* pParent, const PdfObjectList & rlstParents, 
                                        int nIndex, PdfObject* pPage )
 {
@@ -543,7 +702,7 @@ int PdfPagesTree::ChangePagesCount( PdfObject* pPageObj, int nDelta )
 {
     // Increment or decrement inPagesDict's Count by inDelta, and return the new count.
     // Simply return the current count if inDelta is 0.
-    int	cnt = static_cast<int>(pPageObj->GetDictionary().GetKey( "Count" )->GetNumber());
+    int	cnt = GetChildCount( pPageObj );
     if( 0 != nDelta ) 
     {
         cnt += nDelta ;
@@ -555,7 +714,7 @@ int PdfPagesTree::ChangePagesCount( PdfObject* pPageObj, int nDelta )
 
 bool PdfPagesTree::IsEmptyPageNode( PdfObject* pPageNode ) 
 {
-    long lCount = static_cast<long>(pPageNode->GetDictionary().GetKeyAsLong( PdfName("Count"), static_cast<pdf_int64>(0LL) ));
+    long lCount = GetChildCount( pPageNode );
     bool bKidsEmpty = true;
 
     if( pPageNode->GetDictionary().HasKey( PdfName("Kids") ) )
@@ -582,7 +741,7 @@ PdfObject* PdfPagesTree::GetPageNode( int nPageNum, PdfObject* pPagesObject,
 
     PdfArray&	kidsArray = pObj->GetArray();
     size_t	numKids   = kidsArray.size();
-    size_t      kidsCount = pPagesObject->GetDictionary().GetKeyAsLong( "Count", 0 );
+    size_t      kidsCount = GetChildCount( pPagesObject );
 
     // All parents of the page node will be added to this lists,
     // so that the PdfPage can later access inherited attributes
