@@ -16,6 +16,19 @@
  *   License along with this program; if not, write to the                 *
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
+ *                                                                         *
+ *   In addition, as a special exception, the copyright holders give       *
+ *   permission to link the code of portions of this program with the      *
+ *   OpenSSL library under certain conditions as described in each         *
+ *   individual source file, and distribute linked combinations            *
+ *   including the two.                                                    *
+ *   You must obey the GNU General Public License in all respects          *
+ *   for all of the code used other than OpenSSL.  If you modify           *
+ *   file(s) with this exception, you may extend this exception to your    *
+ *   version of the file(s), but you are not obligated to do so.  If you   *
+ *   do not wish to do so, delete this exception statement from your       *
+ *   version.  If you delete this exception statement from all source      *
+ *   files in the program, then also delete it here.                       *
  ***************************************************************************/
 
 #if defined(_MSC_VER)  &&  _MSC_VER <= 1200
@@ -24,6 +37,7 @@
 
 #include <iostream>
 #include <iomanip>
+#include <vector>
 
 #include "PdfPainter.h"
 
@@ -45,6 +59,7 @@
 #include "PdfImage.h"
 #include "PdfMemDocument.h"
 #include "PdfShadingPattern.h"
+#include "PdfTilingPattern.h"
 #include "PdfXObject.h"
 
 
@@ -52,7 +67,7 @@
 
 #define BEZIER_POINTS 13
 
-/* 4/3 * (1-cos 45)/sin 45 = 4/3 * sqrt(2) - 1 */
+/* 4/3 * (1-cos 45,A0(B)/sin 45,A0(B = 4/3 * sqrt(2) - 1 */
 #define ARC_MAGIC    0.552284749f
 #define PI           3.141592654f
 
@@ -69,15 +84,43 @@ static inline void CheckDoubleRange( double val, double min, double max )
     }
 }
 
+static inline unsigned short SwapBytes(unsigned short val)
+{
+	return ((val & 0x00FF) << 8) | ((val & 0xFF00) >> 8);
+}
+
+static inline unsigned short SwapCharBytesIfRequired(pdf_utf16be ch)
+{
+#ifdef PODOFO_IS_LITTLE_ENDIAN
+	return SwapBytes(ch);
+#else
+	return ch;
+#endif
+}
+
+static inline bool IsNewLineChar(pdf_utf16be ch)
+{
+	return SwapCharBytesIfRequired(ch) == '\n';
+}
+
+static inline bool IsSpaceChar(pdf_utf16be ch)
+{
+	return isspace( SwapCharBytesIfRequired(ch) & 0x00FF ) != 0;
+}
+
 PdfPainter::PdfPainter()
 : m_pCanvas( NULL ), m_pPage( NULL ), m_pFont( NULL ), m_nTabWidth( 4 ),
   m_curColor( PdfColor( 0.0, 0.0, 0.0 ) ),
-  m_isTextOpen( false ), m_oss()
+  m_isTextOpen( false ), m_oss(), m_curPath()
 {
     m_oss.flags( std::ios_base::fixed );
     m_oss.precision( clPainterDefaultPrecision );
     PdfLocaleImbue(m_oss);
 
+    m_curPath.flags( std::ios_base::fixed );
+    m_curPath.precision( clPainterDefaultPrecision );
+    PdfLocaleImbue(m_curPath);
+
     lpx  = 
     lpy  = 
     lpx2 = 
@@ -88,6 +131,7 @@ PdfPainter::PdfPainter()
     lcy  = 
     lrx  = 
     lry  = 0.0;
+    currentTextRenderingMode = ePdfTextRenderingMode_Fill;
 }
 
 PdfPainter::~PdfPainter()
@@ -119,13 +163,18 @@ void PdfPainter::SetPage( PdfCanvas* pPage )
     m_pCanvas = pPage ? pPage->GetContentsForAppending()->GetStream() : NULL;
     if ( m_pCanvas ) 
     {
-        m_pCanvas->BeginAppend( false );
+        // GetLength() must be called before BeginAppend()
         if ( m_pCanvas->GetLength() ) 
-        {    
+        {
+            m_pCanvas->BeginAppend( false );
             // there is already content here - so let's assume we are appending
             // as such, we MUST put in a "space" to separate whatever we do.
             m_pCanvas->Append( " " );
         }
+        else
+            m_pCanvas->BeginAppend( false );
+
+        currentTextRenderingMode = ePdfTextRenderingMode_Fill;
     } 
     else 
     {
@@ -148,6 +197,7 @@ void PdfPainter::FinishPage()
 
     m_pCanvas = NULL;
     m_pPage   = NULL;
+    currentTextRenderingMode = ePdfTextRenderingMode_Fill;
 }
 
 void PdfPainter::SetStrokingGray( double g )
@@ -231,6 +281,46 @@ void PdfPainter::SetShadingPattern( const PdfShadingPattern & rPattern )
     m_pCanvas->Append( m_oss.str() );
 }
 
+void PdfPainter::SetStrokingTilingPattern( const PdfTilingPattern & rPattern )
+{
+    PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );    
+
+    this->AddToPageResources( rPattern.GetIdentifier(), rPattern.GetObject()->Reference(), PdfName("Pattern") );
+
+    m_oss.str("");
+    m_oss << "/Pattern CS /" << rPattern.GetIdentifier().GetName() << " SCN" << std::endl;
+    m_pCanvas->Append( m_oss.str() );
+}
+
+void PdfPainter::SetStrokingTilingPattern( const std::string &rPatternName )
+{
+    PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );    
+
+    m_oss.str("");
+    m_oss << "/Pattern CS /" << rPatternName << " SCN" << std::endl;
+    m_pCanvas->Append( m_oss.str() );
+}
+
+void PdfPainter::SetTilingPattern( const PdfTilingPattern & rPattern )
+{
+    PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );    
+
+    this->AddToPageResources( rPattern.GetIdentifier(), rPattern.GetObject()->Reference(), PdfName("Pattern") );
+
+    m_oss.str("");
+    m_oss << "/Pattern cs /" << rPattern.GetIdentifier().GetName() << " scn" << std::endl;
+    m_pCanvas->Append( m_oss.str() );
+}
+
+void PdfPainter::SetTilingPattern( const std::string &rPatternName )
+{
+    PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );    
+
+    m_oss.str("");
+    m_oss << "/Pattern cs /" << rPatternName << " scn" << std::endl;
+    m_pCanvas->Append( m_oss.str() );
+}
+
 void PdfPainter::SetStrokingColor( const PdfColor & rColor )
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );    
@@ -269,6 +359,7 @@ void PdfPainter::SetStrokingColor( const PdfColor & rColor )
 				  " SCN" << std::endl;
             break;
         case ePdfColorSpace_Unknown:
+        case ePdfColorSpace_Indexed:
         {
             PODOFO_RAISE_ERROR( ePdfError_CannotConvertColor );
         }
@@ -316,6 +407,7 @@ void PdfPainter::SetColor( const PdfColor & rColor )
 				  " scn" << std::endl;
 			break;
         case ePdfColorSpace_Unknown:
+        case ePdfColorSpace_Indexed:
         {
             PODOFO_RAISE_ERROR( ePdfError_CannotConvertColor );
         }
@@ -333,32 +425,82 @@ void PdfPainter::SetStrokeWidth( double dWidth )
     m_pCanvas->Append( m_oss.str() );
 }
 
-void PdfPainter::SetStrokeStyle( EPdfStrokeStyle eStyle, const char* pszCustom )
+void PdfPainter::SetStrokeStyle( EPdfStrokeStyle eStyle, const char* pszCustom, bool inverted, double scale, bool subtractJoinCap)
 {
-    const char* pszCurStroke = NULL;
+    bool have = false;
 
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );    
 
-    // TODO: fix the stroking styles
+    m_oss.str("");
+
+    if (eStyle != ePdfStrokeStyle_Custom) {
+        m_oss << "[";
+    }
+
+    if (inverted && eStyle != ePdfStrokeStyle_Solid && eStyle != ePdfStrokeStyle_Custom) {
+       m_oss << "0 ";
+    }
+
     switch( eStyle )
     {
         case ePdfStrokeStyle_Solid:
-            pszCurStroke = "[] 0";
+            have = true;
             break;
         case ePdfStrokeStyle_Dash:
-            pszCurStroke = "[3] 0";
+            have = true;
+            if (scale >= 1.0 - 1e-5 && scale <= 1.0 + 1e-5) {
+                m_oss << "6 2";
+            } else {
+                if (subtractJoinCap) {
+                    m_oss << scale * 2.0 << " " << scale * 2.0;
+                } else {
+                    m_oss << scale * 3.0 << " " << scale * 1.0;
+                }
+            }
             break;
         case ePdfStrokeStyle_Dot:
-            pszCurStroke = "[1] 0";
+            have = true;
+            if (scale >= 1.0 - 1e-5 && scale <= 1.0 + 1e-5) {
+                m_oss << "2 2";
+            } else {
+                if (subtractJoinCap) {
+                    // zero length segments are drawn anyway here
+                    m_oss << 0.001 << " " << 2.0 * scale << " " << 0 << " " << 2.0 * scale;
+                } else {
+                   m_oss << scale * 1.0 << " " << scale * 1.0;
+                }
+            }
             break;
         case ePdfStrokeStyle_DashDot:
-            pszCurStroke = "[3 1 1] 0";
+            have = true;
+            if (scale >= 1.0 - 1e-5 && scale <= 1.0 + 1e-5) {
+                m_oss << "3 2 1 2";
+            } else {
+                if (subtractJoinCap) {
+                    // zero length segments are drawn anyway here
+                    m_oss << scale * 2.0 << " " << scale * 2.0 << " " << 0 << " " << scale * 2.0;
+                } else {
+                    m_oss << scale * 3.0 << " " << scale * 1.0 << " " << scale * 1.0 << " " << scale * 1.0;
+                }
+            }
             break;
         case ePdfStrokeStyle_DashDotDot:
-            pszCurStroke = "[3 1 1 1 1] 0";
+            have = true;
+            if (scale >= 1.0 - 1e-5 && scale <= 1.0 + 1e-5) {
+                m_oss << "3 1 1 1 1 1";
+            } else {
+                if (subtractJoinCap) {
+                    // zero length segments are drawn anyway here
+                    m_oss << scale * 2.0 << " " << scale * 2.0 << " " << 0 << " " << scale * 2.0 << " " << 0 << " " << scale * 2.0;
+                } else {
+                    m_oss << scale * 3.0 << " " << scale * 1.0 << " " << scale * 1.0 << " " << scale * 1.0 << " " << scale * 1.0 << " " << scale * 1.0;
+                }
+            }
             break;
         case ePdfStrokeStyle_Custom:
-            pszCurStroke = pszCustom;
+            have = pszCustom != NULL;
+            if (have)
+                m_oss << pszCustom;
             break;
         default:
         {
@@ -366,13 +508,20 @@ void PdfPainter::SetStrokeStyle( EPdfStrokeStyle eStyle, const char* pszCustom )
         }
     }
 
-    if( !pszCurStroke )
+    if( !have )
     {
         PODOFO_RAISE_ERROR( ePdfError_InvalidStrokeStyle );
     }
     
-    m_oss.str("");
-    m_oss << pszCurStroke << " d" << std::endl;
+    if (inverted && eStyle != ePdfStrokeStyle_Solid && eStyle != ePdfStrokeStyle_Custom) {
+        m_oss << " 0";
+    }
+
+    if (eStyle != ePdfStrokeStyle_Custom) {
+        m_oss << "] 0";
+    }
+
+    m_oss << " d" << std::endl;
     m_pCanvas->Append( m_oss.str() );
 }
 
@@ -390,7 +539,7 @@ void PdfPainter::SetLineJoinStyle( EPdfLineJoinStyle eJoinStyle )
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );    
 
     m_oss.str("");
-    m_oss << static_cast<int>(eJoinStyle) << "j" << std::endl;
+    m_oss << static_cast<int>(eJoinStyle) << " j" << std::endl;
     m_pCanvas->Append( m_oss.str() );
 }
 
@@ -406,6 +555,26 @@ void PdfPainter::SetFont( PdfFont* pFont )
     m_pFont = pFont;
 }
 
+void PdfPainter::SetTextRenderingMode( EPdfTextRenderingMode mode )
+{
+    PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );    
+
+    if (mode == currentTextRenderingMode) {
+        return;
+    }
+
+    currentTextRenderingMode = mode;
+    if (m_isTextOpen)
+        SetCurrentTextRenderingMode();
+}
+
+void PdfPainter::SetCurrentTextRenderingMode( void )
+{
+    PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
+
+    m_oss << (int) currentTextRenderingMode << " Tr" << std::endl;
+}
+
 void PdfPainter::SetClipRect( double dX, double dY, double dWidth, double dHeight )
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );    
@@ -416,6 +585,22 @@ void PdfPainter::SetClipRect( double dX, double dY, double dWidth, double dHeigh
           << dWidth << " "
           << dHeight        
           << " re W n" << std::endl;
+    m_pCanvas->Append( m_oss.str() );
+
+	 m_curPath
+			 << dX << " "
+          << dY << " "
+          << dWidth << " "
+          << dHeight        
+          << " re W n" << std::endl;
+}
+
+void PdfPainter::SetMiterLimit(double value)
+{
+    PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );    
+
+    m_oss.str("");
+    m_oss << value << " M" << std::endl;
     m_pCanvas->Append( m_oss.str() );
 }
 
@@ -423,6 +608,15 @@ void PdfPainter::DrawLine( double dStartX, double dStartY, double dEndX, double
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );    
 
+	 m_curPath.str("");
+    m_curPath
+		    << dStartX << " "
+          << dStartY
+          << " m "
+          << dEndX << " "
+          << dEndY        
+          << " l" << std::endl;
+
     m_oss.str("");
     m_oss << dStartX << " "
           << dStartY
@@ -433,7 +627,7 @@ void PdfPainter::DrawLine( double dStartX, double dStartY, double dEndX, double
     m_pCanvas->Append( m_oss.str() );
 }
 
-void PdfPainter::DrawRect( double dX, double dY, double dWidth, double dHeight,
+void PdfPainter::Rectangle( double dX, double dY, double dWidth, double dHeight,
                            double dRoundX, double dRoundY )
 { 
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );    
@@ -454,58 +648,27 @@ void PdfPainter::DrawRect( double dX, double dY, double dWidth, double dHeight,
         CubicBezierTo(x + rx * b, y + h, x, y + h - ry * b, x, y + h - ry);
         LineTo(x, y + ry);
         CubicBezierTo(x, y + ry * b, x + rx * b, y, x + rx, y);
-        m_pCanvas->Append( "S\n" );
     } 
     else 
     {
-        m_oss.str("");
-        m_oss << dX << " "
+        m_curPath 
+				  << dX << " "
               << dY << " "
               << dWidth << " "
               << dHeight        
-              << " re S" << std::endl;
-        m_pCanvas->Append( m_oss.str() );
-    }
-}
-
-void PdfPainter::FillRect( double dX, double dY, double dWidth, double dHeight,
-                           double dRoundX, double dRoundY )
-{
-    PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
+              << " re" << std::endl;
 
     m_oss.str("");
-
-    if ( static_cast<int>(dRoundX) || static_cast<int>(dRoundY) ) 
-    {
-        double    x = dX, y = dY, 
-            w = dWidth, h = dHeight,
-            rx = dRoundX, ry = dRoundY;
-        double b = 0.4477f;
-
-        MoveTo(x + rx, y);
-        LineTo(x + w - rx, y);
-        CubicBezierTo(x + w - rx * b, y, x + w, y + ry * b, x + w, y + ry);
-        LineTo(x + w, y + h - ry);
-        CubicBezierTo(x + w, y + h - ry * b, x + w - rx * b, y + h, x + w - rx, y + h);
-        LineTo(x + rx, y + h);
-        CubicBezierTo(x + rx * b, y + h, x, y + h - ry * b, x, y + h - ry);
-        LineTo(x, y + ry);
-        CubicBezierTo(x, y + ry * b, x + rx * b, y, x + rx, y);
-        m_pCanvas->Append( "f\n" );
-    }
-    else 
-    {
         m_oss << dX << " "
             << dY << " "
             << dWidth << " "
             << dHeight        
-            << " re f" << std::endl;
-    }
-
+              << " re" << std::endl;
     m_pCanvas->Append( m_oss.str() );
 }
+}
 
-void PdfPainter::DrawEllipse( double dX, double dY, double dWidth, double dHeight )
+void PdfPainter::Ellipse( double dX, double dY, double dWidth, double dHeight )
 {
     double dPointX[BEZIER_POINTS];
     double dPointY[BEZIER_POINTS];
@@ -515,6 +678,10 @@ void PdfPainter::DrawEllipse( double dX, double dY, double dWidth, double dHeigh
 
     ConvertRectToBezier( dX, dY, dWidth, dHeight, dPointX, dPointY );
 
+    m_curPath
+			 << dPointX[0] << " "
+          << dPointY[0]
+          << " m" << std::endl;
 
     m_oss.str("");
     m_oss << dPointX[0] << " "
@@ -523,6 +690,15 @@ void PdfPainter::DrawEllipse( double dX, double dY, double dWidth, double dHeigh
 
     for( i=1;i<BEZIER_POINTS; i+=3 )
     {
+        m_curPath
+				  << dPointX[i] << " "
+              << dPointY[i] << " "
+              << dPointX[i+1] << " "
+              << dPointY[i+1] << " "
+              << dPointX[i+2] << " "
+              << dPointY[i+2]    
+              << " c" << std::endl;
+
         m_oss << dPointX[i] << " "
               << dPointY[i] << " "
               << dPointX[i+1] << " "
@@ -533,63 +709,9 @@ void PdfPainter::DrawEllipse( double dX, double dY, double dWidth, double dHeigh
     }
 
     m_pCanvas->Append( m_oss.str() );
-    m_pCanvas->Append( "S\n" );
 }
 
-void PdfPainter::FillEllipse( double dX, double dY, double dWidth, double dHeight )
-{
-    double dPointX[BEZIER_POINTS];
-    double dPointY[BEZIER_POINTS];
-    int    i;
-
-    PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
-
-    ConvertRectToBezier( dX, dY, dWidth, dHeight, dPointX, dPointY );
-
-    m_oss.str("");
-    m_oss << dPointX[0] << " "
-          << dPointY[0]
-          << " m" << std::endl;
-
-    for( i=1;i<BEZIER_POINTS; i+=3 )
-    {
-        m_oss << dPointX[i] << " "
-              << dPointY[i] << " "
-              << dPointX[i+1] << " "
-              << dPointY[i+1] << " "
-              << dPointX[i+2] << " "
-              << dPointY[i+2]    
-              << " c" << std::endl;
-    }
-
-    m_pCanvas->Append( m_oss.str() );
-    m_pCanvas->Append( "f\n" );
-}
-
-void PdfPainter::FillCircle( double dX, double dY, double dRadius )
-{
-    PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
-
-    /* draw four Bezier curves to approximate a circle */
-    MoveTo( dX + dRadius, dY );
-    CubicBezierTo( dX + dRadius, dY + dRadius*ARC_MAGIC,
-             dX + dRadius*ARC_MAGIC, dY + dRadius,
-             dX, dY + dRadius );
-    CubicBezierTo( dX - dRadius*ARC_MAGIC, dY + dRadius,
-            dX - dRadius, dY + dRadius*ARC_MAGIC,
-            dX - dRadius, dY );
-    CubicBezierTo( dX - dRadius, dY - dRadius*ARC_MAGIC,
-            dX - dRadius*ARC_MAGIC, dY - dRadius,
-            dX, dY - dRadius );
-    CubicBezierTo( dX + dRadius*ARC_MAGIC, dY - dRadius,
-            dX + dRadius, dY - dRadius*ARC_MAGIC,
-            dX + dRadius, dY );
-    Close();
-
-    m_pCanvas->Append( "f\n" );
-}
-
-void PdfPainter::DrawCircle( double dX, double dY, double dRadius )
+void PdfPainter::Circle( double dX, double dY, double dRadius )
 {
     if( !m_pCanvas )
     {
@@ -611,8 +733,6 @@ void PdfPainter::DrawCircle( double dX, double dY, double dRadius )
             dX + dRadius, dY - dRadius*ARC_MAGIC,
             dX + dRadius, dY );
     Close();
-
-    m_pCanvas->Append( "S\n" );
 }
 
 void PdfPainter::DrawText( double dX, double dY, const PdfString & sText )
@@ -687,6 +807,10 @@ void PdfPainter::DrawText( double dX, double dY, const PdfString & sText, long l
           << " "  << m_pFont->GetFontSize()
           << " Tf" << std::endl;
 
+    if (currentTextRenderingMode != ePdfTextRenderingMode_Fill) {
+        SetCurrentTextRenderingMode();
+    }
+
     //if( m_pFont->GetFontScale() != 100.0F ) - this value is kept between text blocks
     m_oss << m_pFont->GetFontScale() << " Tz" << std::endl;
 
@@ -727,6 +851,10 @@ void PdfPainter::BeginText( double dX, double dY )
           << " "  << m_pFont->GetFontSize()
           << " Tf" << std::endl;
 
+    if (currentTextRenderingMode != ePdfTextRenderingMode_Fill) {
+        SetCurrentTextRenderingMode();
+    }
+
     //if( m_pFont->GetFontScale() != 100.0F ) - this value is kept between text blocks
     m_oss << m_pFont->GetFontScale() << " Tz" << std::endl;
 
@@ -795,7 +923,7 @@ void PdfPainter::EndText()
 }
 
 void PdfPainter::DrawMultiLineText( double dX, double dY, double dWidth, double dHeight, const PdfString & rsText, 
-                                    EPdfAlignment eAlignment, EPdfVerticalAlignment eVertical )
+                                    EPdfAlignment eAlignment, EPdfVerticalAlignment eVertical, bool bClip )
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
 
@@ -805,17 +933,20 @@ void PdfPainter::DrawMultiLineText( double dX, double dY, double dWidth, double
     }
 
     // Peter Petrov 25 September 2008
-    m_pFont->EmbedFont();
+    //m_pFont->EmbedFont();
     
     if( dWidth <= 0.0 || dHeight <= 0.0 ) // nonsense arguments
         return;
 
     this->Save();
-    this->SetClipRect( dX, dY, dWidth, dHeight );
+    if( bClip ) 
+    {
+        this->SetClipRect( dX, dY, dWidth, dHeight );
+    }
 
     PdfString   sString  = this->ExpandTabs( rsText, rsText.GetCharacterLength() );
 
-    std::vector<TLineElement> vecLines = GetMultiLineTextAsLines(dWidth, sString);
+	std::vector<PdfString> vecLines = GetMultiLineTextAsLines(dWidth, sString);
 
     // Do vertical alignment
     switch( eVertical ) 
@@ -831,19 +962,19 @@ void PdfPainter::DrawMultiLineText( double dX, double dY, double dWidth, double
             break;
     }
 
-    std::vector<TLineElement>::const_iterator it = vecLines.begin();
+    std::vector<PdfString>::const_iterator it = vecLines.begin();
     while( it != vecLines.end() )
     {
         dY -= m_pFont->GetFontMetrics()->GetLineSpacing();
-        if( (*it).pszStart )
-            this->DrawTextAligned( dX, dY, dWidth, PdfString( (*it).pszStart, (*it).lLen ), eAlignment );
+		if( (*it).GetCharacterLength() )
+            this->DrawTextAligned( dX, dY, dWidth, *it, eAlignment );
 
         ++it;
     }
     this->Restore();
 }
 
-std::vector<TLineElement> PdfPainter::GetMultiLineTextAsLines( double dWidth, const PdfString & rsText)
+std::vector<PdfString> PdfPainter::GetMultiLineTextAsLines( double dWidth, const PdfString & rsText)
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
 
@@ -851,52 +982,55 @@ std::vector<TLineElement> PdfPainter::GetMultiLineTextAsLines( double dWidth, co
     {
         PODOFO_RAISE_ERROR( ePdfError_InvalidHandle );
     }
-
-    std::vector<TLineElement> vecLines;
-
+     
     if( dWidth <= 0.0 ) // nonsense arguments
-        return vecLines;
-
-    TLineElement              tLine;
+	    return std::vector<PdfString>();
     
-    tLine.pszStart       = rsText.GetString();
-    
-    const char* pszCurrentCharacter   = tLine.pszStart;
-    const char* pszStartOfCurrentWord  = tLine.pszStart;
+    if( rsText.GetCharacterLength() == 0 ) // empty string
+        return std::vector<PdfString>(1, rsText);
+	        
+    // We will work with utf16 encoded string because it allows us 
+    // fast and easy individual characters access    
+    const std::string& stringUtf8 = rsText.GetStringUtf8();
+    std::vector<pdf_utf16be> stringUtf16(stringUtf8.length() + 1, 0);
+	assert(stringUtf16.size() > 0);	
+    const pdf_long converted = PdfString::ConvertUTF8toUTF16(
+	    reinterpret_cast<const pdf_utf8*>(stringUtf8.c_str()), &stringUtf16[0], stringUtf16.size());
+	//const pdf_long len = rsText.GetCharacterLength();
+    assert( converted == (rsText.GetCharacterLength() + 1) );
 
+	const pdf_utf16be* const stringUtf16Begin = &stringUtf16[0];
+    const pdf_utf16be* pszLineBegin = stringUtf16Begin;
+    const pdf_utf16be* pszCurrentCharacter = stringUtf16Begin;
+    const pdf_utf16be* pszStartOfCurrentWord  = stringUtf16Begin;
     bool startOfWord = true;
-
     double dCurWidthOfLine = 0.0;
+	std::vector<PdfString> vecLines;
 
     // do simple word wrapping
     while( *pszCurrentCharacter ) 
     {
-        if( *pszCurrentCharacter == '\n' ) // hard-break!
+        if( IsNewLineChar( *pszCurrentCharacter ) ) // hard-break! 
         {
-            tLine.lLen = pszCurrentCharacter - tLine.pszStart;
-            vecLines.push_back( tLine );
-
+            vecLines.push_back( PdfString( pszLineBegin, pszCurrentCharacter - pszLineBegin ) );
             ++pszCurrentCharacter; // skip the line feed
-
-            tLine.pszStart = pszCurrentCharacter;
+            pszLineBegin = pszCurrentCharacter;
             startOfWord = true;
             dCurWidthOfLine = 0.0;
         }
-        else if( isspace( static_cast<unsigned int>(static_cast<unsigned char>(*pszCurrentCharacter)) ) /*|| 
-                 ispunct( static_cast<unsigned int>(static_cast<unsigned char>(*pszCurrentCharacter)) )*/)
+        else if( IsSpaceChar( *pszCurrentCharacter ) )
         {
             if( dCurWidthOfLine > dWidth )
             {
                 // The previous word does not fit in the current line.
                 // -> Move it to the next one.
-                tLine.lLen = pszStartOfCurrentWord - tLine.pszStart;
-                vecLines.push_back( tLine );
-
-                tLine.pszStart = pszStartOfCurrentWord;
+                vecLines.push_back( PdfString( pszLineBegin, pszStartOfCurrentWord - pszLineBegin ) );
+                pszLineBegin = pszStartOfCurrentWord;
 
                 if (!startOfWord)
                 {
-                    dCurWidthOfLine = m_pFont->GetFontMetrics()->StringWidth( pszStartOfCurrentWord, pszCurrentCharacter-pszStartOfCurrentWord );
+                    dCurWidthOfLine = m_pFont->GetFontMetrics()->StringWidth( 
+						pszStartOfCurrentWord, pszCurrentCharacter - pszStartOfCurrentWord );
                 }
                 else
                 {
@@ -904,9 +1038,8 @@ std::vector<TLineElement> PdfPainter::GetMultiLineTextAsLines( double dWidth, co
                 }
             }
             else 
-            {
-           
-                dCurWidthOfLine += m_pFont->GetFontMetrics()->CharWidth( *pszCurrentCharacter );
+            {           
+                dCurWidthOfLine += m_pFont->GetFontMetrics()->UnicodeCharWidth( SwapCharBytesIfRequired( *pszCurrentCharacter ) );
             }
 
             startOfWord = true;
@@ -920,58 +1053,48 @@ std::vector<TLineElement> PdfPainter::GetMultiLineTextAsLines( double dWidth, co
             }
             //else do nothing
 
-            if ((dCurWidthOfLine + m_pFont->GetFontMetrics()->CharWidth( *pszCurrentCharacter )) > dWidth)
+            if ((dCurWidthOfLine + m_pFont->GetFontMetrics()->UnicodeCharWidth( SwapCharBytesIfRequired( *pszCurrentCharacter ))) > dWidth)
             {
-                if ( tLine.pszStart == pszStartOfCurrentWord )
+                if ( pszLineBegin == pszStartOfCurrentWord )
                 {
                     // This word takes up the whole line.
-                    // Put as much as possible on this line.
-                    tLine.lLen = pszCurrentCharacter - tLine.pszStart;
-                    vecLines.push_back( tLine );
-
-                    tLine.pszStart = pszCurrentCharacter;
+                    // Put as much as possible on this line.                    
+                    vecLines.push_back( PdfString( pszLineBegin, pszCurrentCharacter - pszLineBegin ) );
+                    pszLineBegin = pszCurrentCharacter;
                     pszStartOfCurrentWord = pszCurrentCharacter;
-
-                    dCurWidthOfLine = m_pFont->GetFontMetrics()->CharWidth( *pszCurrentCharacter );
+                    dCurWidthOfLine = m_pFont->GetFontMetrics()->UnicodeCharWidth( SwapCharBytesIfRequired( *pszCurrentCharacter ) );
                 }
                 else
                 {
                     // The current word does not fit in the current line.
-                    // -> Move it to the next one.
-                    tLine.lLen = pszStartOfCurrentWord - tLine.pszStart;
-                    vecLines.push_back( tLine );
-
-                    tLine.pszStart = pszStartOfCurrentWord;
-
+                    // -> Move it to the next one.                    
+                    vecLines.push_back( PdfString( pszLineBegin, pszStartOfCurrentWord - pszLineBegin ) );
+                    pszLineBegin = pszStartOfCurrentWord;
                     dCurWidthOfLine = m_pFont->GetFontMetrics()->StringWidth( pszStartOfCurrentWord, (pszCurrentCharacter-pszStartOfCurrentWord) + 1 );
                 }
             }
             else 
             {
-                dCurWidthOfLine += m_pFont->GetFontMetrics()->CharWidth( *pszCurrentCharacter );
+                dCurWidthOfLine += m_pFont->GetFontMetrics()->UnicodeCharWidth( SwapCharBytesIfRequired( *pszCurrentCharacter ) );
             }
         }
         ++pszCurrentCharacter;
     }
 
-    if( pszCurrentCharacter-tLine.pszStart > 0 ) 
+    if( (pszCurrentCharacter - pszLineBegin) > 0 ) 
     {
         if( dCurWidthOfLine > dWidth )
         {
             // The previous word does not fit in the current line.
             // -> Move it to the next one.
-            tLine.lLen = pszStartOfCurrentWord - tLine.pszStart;
-            vecLines.push_back( tLine );
-
-            tLine.pszStart = pszStartOfCurrentWord;
-
+            vecLines.push_back( PdfString( pszLineBegin, pszStartOfCurrentWord - pszLineBegin) );
+            pszLineBegin = pszStartOfCurrentWord;
         }
         //else do nothing
 
-        if( pszCurrentCharacter-tLine.pszStart > 0 ) 
+        if( pszCurrentCharacter - pszLineBegin > 0 ) 
         {
-            tLine.lLen = pszCurrentCharacter - tLine.pszStart;
-            vecLines.push_back( tLine );
+            vecLines.push_back( PdfString( pszLineBegin, pszCurrentCharacter - pszLineBegin) );
         }
         //else do nothing
     }
@@ -989,7 +1112,7 @@ void PdfPainter::DrawTextAligned( double dX, double dY, double dWidth, const Pdf
     }
 
     // Peter Petrov 25 Septemer 2008
-    m_pFont->EmbedFont();
+    //m_pFont->EmbedFont();
 
     if( dWidth <= 0.0 ) // nonsense arguments
         return;
@@ -1152,6 +1275,8 @@ void PdfPainter::ClosePath()
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
 
+	 m_curPath << "h" << std::endl;
+
     m_pCanvas->Append( "h\n" );
 }
 
@@ -1159,6 +1284,11 @@ void PdfPainter::LineTo( double dX, double dY )
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
     
+	 m_curPath
+			 << dX << " "
+          << dY
+          << " l" << std::endl;
+
     m_oss.str("");
     m_oss << dX << " "
           << dY
@@ -1170,6 +1300,11 @@ void PdfPainter::MoveTo( double dX, double dY )
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
     
+	 m_curPath
+			 << dX << " "
+          << dY
+          << " m" << std::endl;
+
     m_oss.str("");
     m_oss << dX << " "
           << dY
@@ -1181,6 +1316,15 @@ void PdfPainter::CubicBezierTo( double dX1, double dY1, double dX2, double dY2,
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
 
+	 m_curPath
+			 << dX1 << " "
+          << dY1 << " "
+          << dX2 << " "
+          << dY2 << " "
+          << dX3 << " "
+          << dY3 
+          << " c" << std::endl;
+
     m_oss.str("");
     m_oss << dX1 << " "
           << dY1 << " "
@@ -1192,7 +1336,7 @@ void PdfPainter::CubicBezierTo( double dX1, double dY1, double dX2, double dY2,
     m_pCanvas->Append( m_oss.str() );
 }
 
-void PdfPainter::HorizonalLineTo( double inX )
+void PdfPainter::HorizontalLineTo( double inX )
 {
     LineTo( inX, lpy3 );
 }
@@ -1360,7 +1504,7 @@ void PdfPainter::ArcTo( double inX, double inY, double inRadiusX, double inRadiu
 }
 
 // Peter Petrov 5 January 2009 was delivered from libHaru
-bool PdfPainter::DrawArc(double dX, double dY, double dRadius, double dAngle1, double dAngle2)
+bool PdfPainter::Arc(double dX, double dY, double dRadius, double dAngle1, double dAngle2)
 {
     bool cont_flg = false;
 
@@ -1458,32 +1602,62 @@ bool PdfPainter::InternalArc(
 void PdfPainter::Close()
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
-    
+
+    m_curPath << "h" << std::endl;
+
     m_pCanvas->Append( "h\n" );
 }
 
 void PdfPainter::Stroke()
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
-    
+
+    m_curPath.str("");
+
     m_pCanvas->Append( "S\n" );
 }
 
-void PdfPainter::Fill()
+void PdfPainter::Fill(bool useEvenOddRule)
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
-    
-    m_pCanvas->Append( "f\n" );
+
+    m_curPath.str("");
+
+    if (useEvenOddRule)
+        m_pCanvas->Append( "f*\n" );
+    else
+        m_pCanvas->Append( "f\n" );
+}
+
+void PdfPainter::FillAndStroke(bool useEvenOddRule)
+{
+    PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
+
+    m_curPath.str("");
+
+    if (useEvenOddRule)
+        m_pCanvas->Append( "B*\n" );
+    else
+        m_pCanvas->Append( "B\n" );
 }
 
 void PdfPainter::Clip( bool useEvenOddRule )
 {
     PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
     
-	if ( useEvenOddRule )
-	    m_pCanvas->Append( "W* n\n" );
-	else
-	    m_pCanvas->Append( "W n\n" );
+    if ( useEvenOddRule )
+        m_pCanvas->Append( "W* n\n" );
+    else
+        m_pCanvas->Append( "W n\n" );
+}
+
+void PdfPainter::EndPath(void)
+{
+    PODOFO_RAISE_LOGIC_IF( !m_pCanvas, "Call SetPage() first before doing drawing operations." );
+
+    m_curPath << "n" << std::endl;
+
+    m_pCanvas->Append( "n\n" );
 }
 
 void PdfPainter::Save()
